# =============================================================================================================
# *
# * Copyright (c) Mike
# *
# * File Name: simpleCPUv1d.m4
# *
# * Version: V1.0
# *
# * Release Date:
# *
# * Author(s): M.Freeman
# *
# * Description: macros
# *
# * Conditions of Use: THIS CODE IS COPYRIGHT AND IS SUPPLIED "AS IS" WITHOUT WARRANTY OF ANY KIND, INCLUDING,
# *                    BUT NOT LIMITED TO, ANY IMPLIED WARRANTY OF MERCHANTABILITY AND FITNESS FOR A
# *                    PARTICULAR PURPOSE.
# *
# * Notes:
# *
# =============================================================================================================
#    unary + and -
#    ** or ^ (exponentiation)
#    * / % (modulus)
#    + -
#    == != < <= > >=
#    !(not)
#    & or && (logical AND)
#    | or || (logical OR)
# ------
# - IO -
# ------
# memory map
# ----------
#                          3210
# CE0 : UART   : 1111 1111 111X
# CE1 : GPIO A : 1111 1111 110X
# CE2 : ROTARY : 1111 1111 101X
# CE3 : SNES   : 1111 1111 100X
# CE4 : PS2    : 1111 1111 011X
# CE5 : NU     : 1111 1111 010X
# CE6 : VGA    : 1111 1111 0XXX
# --------
# - GAME -
# --------
# ------------
# - CONSTANT -
# ------------
# -----------
# - COLOURS -
# -----------
#             XXRR GGBB 
# RED         0011 0000 = 0x30
# GREEN       0000 1100 = 0x0C
# BLUE        0000 0011 = 0x03
# YELLOW      0011 1100 = 0x3C
# ORANGE      0011 0100 = 0x34
# TURQUOISE   0000 1111 = 0x0F
# LIGHT_BLUE  0000 0111 = 0x07
# LIGHT_GREEN 0000 1101 = 0x0D
# LIME_GREEN  0001 1100 = 0x1C
# PURPLE      0001 0011 = 0x13
# PINK        0011 0011 = 0x33
# SCARLET     0011 0001 = 0x31
# GREY        0001 0101 = 0x15
# LIGHT_GREY  0010 1010 = 0x2A
# BLACK       0000 0000 = 0x00
# WHITE       0011 1111 = 0xFF
# -----------------------
# - VIDEO COMMAND CODES -
# -----------------------
# ----------------------------
# - CHARACTER TILE ADDRESSES -
# ----------------------------
# -----------------------------------------
# - SPACE INVADERS GRAPHIC TILE ADDRESSES -
# -----------------------------------------
# ------------------------------
# - OXO GRAPHIC TILE ADDRESSES -
# ------------------------------
# -----------
# - GENERAL -
# -----------
# VAR VAL
# 1   2
# VAR, MASK, ADDR
# 1    2     3
# VAR, VAL, ADDR
# 1    2    3
# VAR ADDR=0 ADDR=1 ADDR=2 ...
# 1   2      3      4  
# ------------------
# - GENERATE LABEL -
# ------------------
# -----------
# - CONVERT -
# -----------
# covert_XY_2_address : convert X Y position values into pixel address
# --------------------------------------------------------------------
# ---------------------------
# - RESERVE SPACE IN MEMORY -
# ---------------------------
# -------------
# - BIT ARRAY -
# -------------
# VARIABLES
# ---------
# RA - bit_array_data
# RB - bit_array_offset
# RC - bit_array_address
# create bit array( SIZE )
#                   1 
# get_array_bit( BASE, INDEX )
#                1     2
# get_array_bit( BASE )
#                1    
# set_array_bit( BASE, INDEX )
#                1     2
# set_array_bit( BASE )
#                1 
# clr_array_bit( BASE, INDEX )
#                1     2
# clr_array_bit( BASE )
#                1 
# -----------------------
# - MOVE ADDRESS / DATA -
# -----------------------
# movea : move 12bit address into register
# ---------------------------------------- 
# moved : move 16 bit data into register
# --------------------------------------
# --------------------
# - ROTATE AND SHIFT -
# --------------------
# ---------
# - SLEEP -
# ---------
# -----------
# - SPRITES -
# -----------
# VISIBLE, AUTO TILE_INC TILE_NUMBER, TILE_ADDR_0, TILE_ADDR_1, TILE_ADDR_2, TILE_ADDR_3, TILE_ADDR_4, TILE_ADDR_5, TILE_ADDR_6, TILE_ADDR_7, 
# 1        2    3        4            5            6            7            8            9            10           11           12       
# BG COLOUR, FG COLOUR, XPOS, YPOS, XOFFSET, YOFFSET, HB_XMAX, HB_XMIN, HB_YMAX, HB_YMIN, LT_XMAX, LT_XMIN, LT_YMAX, LT_YMIN,
# 13         14         15    16    17       18       19       20       21       22       23       24       25       26
# 0  : VISIBLE : AUTO : TILE_INC : TILE_NUMBER (1:1:1:3)
# 1  : TILE_ADDRESS_0
# 2  : TILE_ADDRESS_1
# 3  : TILE_ADDRESS_2
# 4  : TILE_ADDRESS_3
# 5  : TILE_ADDRESS_4
# 6  : TILE_ADDRESS_5
# 7  : TILE_ADDRESS_6
# 8  : TILE_ADDRESS_7
# 9  : BG COLOUR : FG COLOUR		(8:8)
# 10 : XPOS : YPOS					(8:8)
# 11 : XOFFSET : YOFFSET			(8:8)
# 12 : HB_XMAX : HB_XMIN			(8:8)
# 13 : HB_YMAX : HB_YMIN		    (8:8)
# 14 : LT_XMAX : LT_XMIN			(8:8)
# 15 : LT_YMAX : LT_YMIN		    (8:8)
# VISIBLE     : 0 = not visible, 1 = visible
# AUTO        : 0 = direction fixed, 1 = direction reversed (limit collisions)
# TILE_INC    : 0 = fixed tile number, 1 = incrementing tile number
# TILE_NUMBER : select tile address 0 - 7, 3bit value
# BG_COLOUR   : background colour, 8bit value
# FG_COLOUR   : foreground colour, 8bit value
# XPOS        : x position of top left pixel of tile, 8bit value
# YPOS        : y position of top left pixel of tile, 8bit value
# XOFFSET     : x offset, 8bit value
# YOFFSET     : y offset, 8bit value
# HB_XMAX     : hit box, x max, 8bit value
# HB_XMIN     : hit box, x min, 8bit value
# HB_YMAX     : hit box, y max, 8bit value
# HB_YMIN     : hit box, y min, 8bit value
# LT_XMAX     : limit, x max, 8bit value
# LT_XMIN     : limit, x min, 8bit value
# LT_YMAX     : limit, y max, 8bit value
# LT_YMIN     : limit, y min, 8bit value
#    32 16 8  421
# XX V: A: I: TTT
#
# initialize_sprite (initialise memory before run time)
# ------------------------------------------------
#
# draw_sprite
# -----------
#
# SPRITE_BASE_ADDR, SPRITE_NUMBER
# 1                 2                                                     
#
# overlay_sprite
# --------------
#
# SPRITE_BASE_ADDR, SPRITE_NUMBER
# 1                 2     
# move_sprite
# -----------
#
# SPRITE_BASE_ADDR, SPRITE_NUMBER
# 1                 2     
#
# clear_sprite
# -----------
#
# SPRITE_BASE_ADDR, SPRITE_NUMBER
# 1                 2     
#
# collision_sprite
# ----------------
#
# SPRITE_BASE_ADDR_A, SPRITE_NUMBER_A, SPRITE_BASE_ADDR_B, SPRITE_NUMBER_B 
# 1                   2                3                   4                     
#
#
# turn_off_sprite
# ---------------
#
# SPRITE_BASE_ADDR, SPRITE_NUMBER
# 1                 2                           
#
#    32 16 8  421
# XX V: A: I: TTT
#
#
# turn_on_sprite
# ---------------
#
# SPRITE_BASE_ADDR, SPRITE_NUMBER
# 1                 2                            
#
#    32 16 8  421
# XX V: A: I: TTT
#
#
# set_sprite
# ----------
#
# SPRITE_BASE_ADDR, SPRITE_NUMBER, VAL
# 1                 2              3              
#
#    32 16 8  421
# XX V: A: I: TTT
#
#
# increment_sprite
# ---------------
#
# SPRITE_BASE_ADDR, SPRITE_NUMBER
# 1                 2                            
#
#    32 16 8  421
# XX V: A: I: TTT
#
#
# get_sprite
# ----------
#
# SPRITE_BASE_ADDR, SPRITE_NUMBER
# 1                 2   
#
# position_sprite
# ---------------
#
# SPRITE_BASE_ADDR, SPRITE_NUMBER, XVAL, YVAL
# 1                 2              3     4              
#
#    32 16 8  421
# XX V: A: I: TTT
#
#
# offset_sprite
# -------------
#
# SPRITE_BASE_ADDR, SPRITE_NUMBER, XVAL, YVAL
# 1                 2              3     4              
#
#    32 16 8  421
# XX V: A: I: TTT
#
#
# delete_sprite (at run time)
# ---------------------------
# SPRITE_BASE_ADDR, SPRITE_NUMBER
# 1                 2                
#
# clear_screen
# ------------
#
# draw_box
# --------
# COLOUR, X_TOP, Y_TOP, X_BOTTOM, Y_BOTTOM
# 1       2      3      4         5
#             1   2  3  4  5 
#  draw_box( RED, 6, 2, 7, 3 )
#
# copy_box
# --------
# SRC_X_TOP, SRC_Y_TOP, SRC_X_BOTTOM, SRC_Y_BOTTOM, DEST_X_TOP, DEST_Y_TOP
# 1          2          3             4             5           6
#
# draw_tile
# ---------
# COLOUR, X, Y, CHAR
# 1       2  3  4    
#
# overlay_tile
# ------------
# COLOUR, X, Y, CHAR
# 1       2  3  4    
000  move ra 0
001  store ra 2116
002  move ra 0
003  store ra 2121
004  lhi ra 37
005  or ra 127
006  store ra 2122
007  call 1351 # clear screen
008  call 27 # setup game variable
009  call 333 # 406 snake
010  call 354 # 406 354
011  call 195 # read keyboard, 195 state and display
012  call 913 # 911
013  load ra 2615 # is 354 eaten flag set
014  and ra 0xff
015  jumpz 20 # no 20 2616 flag
016  move ra 0 # clr flag
017  store ra 2615
018  call 540 # inc score
019  call 354 # 406 new 354
020  load ra 2616 # is finished flag set
021  and ra 0xff
022  jumpz 11 # no 11
023  call 418 # 406 game over message
024  call 922 # wait
025  jump 0 # restart
026  jump 26
027  move ra 0 # zero main variables
028  store ra 2721
029  store ra 2824
030  store ra 2386
031  store ra 2407
032  store ra 2619
033  store ra 2616
034  store ra 2615
035  lhi ra 10
036  or ra 161 # set tail pointer
037  store ra 2611
038  lhi ra 10
039  or ra 60 # set list address
040  store ra 2613
041  move ra 80 # starting x pos
042  store ra 2617
043  move ra 0
044  store ra 2114 # 11 2114
045  load ra 2617
046  store ra 2614
047  call 2035
048  load ra 2617 # snake segments four pixels in size
049  add ra 4
050  store ra 2617
051  load ra 2114
052  add ra 1
053  store ra 2114
054  sub ra 6 # repeat for snake length
055  jumpnz 45
056  lhi ra 11
057  or ra 8 # set index address
058  store ra 2611
059  lhi ra 10
060  or ra 163 # set list address
061  store ra 2613
062  move ra 72 # starting y pos
063  store ra 2618
064  move ra 0
065  store ra 2114
066  load ra 2618
067  store ra 2614
068  call 2035
069  load ra 2114 # repeat for snake length
070  add ra 1
071  store ra 2114
072  sub ra 6
073  jumpnz 66
074  move ra 0x0f
075  store ra 2116
076  lhi ra 5
077  or ra 82
078  store ra 2121
079  lhi ra 0
080  or ra 75
081  store ra 2122
082  lhi ra 0
083  or ra 3
084  store ra 2123
085  call 1374
086  move ra 0x0f
087  store ra 2116
088  lhi ra 35
089  or ra 162
090  store ra 2121
091  lhi ra 0
092  or ra 75
093  store ra 2122
094  lhi ra 0
095  or ra 3
096  store ra 2123
097  call 1374
098  move ra 0x0f
099  store ra 2116
100  lhi ra 5
101  or ra 82
102  store ra 2121
103  lhi ra 0
104  or ra 1
105  store ra 2122
106  lhi ra 0
107  or ra 98
108  store ra 2123
109  call 1374
110  move ra 0x0f
111  store ra 2116
112  lhi ra 5
113  or ra 156
114  store ra 2121
115  lhi ra 0
116  or ra 1
117  store ra 2122
118  lhi ra 0
119  or ra 98
120  store ra 2123
121  call 1374
122  move ra 0x2a
123  store ra 2116
124  move ra 3
125  store ra 2117
126  move ra 9
127  store ra 2119
128  lhi rd 39
129  or rd 76
130  call 1291
131  move ra 0x2a
132  store ra 2116
133  move ra 9
134  store ra 2117
135  move ra 9
136  store ra 2119
137  lhi rd 39
138  or rd 12
139  call 1291
140  move ra 0x2a
141  store ra 2116
142  move ra 16
143  store ra 2117
144  move ra 9
145  store ra 2119
146  lhi rd 39
147  or rd 60
148  call 1291
149  move ra 0x2a
150  store ra 2116
151  move ra 24
152  store ra 2117
153  move ra 9
154  store ra 2119
155  lhi rd 39
156  or rd 72
157  call 1291
158  move ra 0x2a
159  store ra 2116
160  move ra 32
161  store ra 2117
162  move ra 9
163  store ra 2119
164  lhi rd 39
165  or rd 20
166  call 1291
167  move ra 0x2a
168  store ra 2116
169  move ra 43
170  store ra 2117
171  move ra 9
172  store ra 2119
173  lhi rd 38
174  or rd 64
175  call 1291
176  move ra 0x2a
177  store ra 2116
178  move ra 51
179  store ra 2117
180  move ra 9
181  store ra 2119
182  lhi rd 38
183  or rd 64
184  call 1291
185  move ra 0x2a
186  store ra 2116
187  move ra 59
188  store ra 2117
189  move ra 9
190  store ra 2119
191  lhi rd 38
192  or rd 64
193  call 1291
194  ret
195  load ra 2620
196  store ra 2722 # buffer current x pos
197  load ra 2723
198  store ra 2825 # buffer current y pos
199  call 1689 # read last 2386 pressed
200  load ra 2386 # 20 last 2386 pressed
201  sub ra 0x1b
202  jumpz 214
203  sub ra 1 # 0x1c
204  jumpz 218
205  sub ra 1 # 0x1d
206  jumpz 222
207  sub ra 6 # 0x23
208  jumpz 210
209  jump 353
210  load ra 2722 #
211  add ra 4
212  store ra 2722
213  jump 225
214  load ra 2825 #
215  add ra 4
216  store ra 2825
217  jump 225
218  load ra 2722 #
219  sub ra 4
220  store ra 2722
221  jump 225
222  load ra 2825 #
223  sub ra 4
224  store ra 2825
225  move ra 0 # has snake hit itself check
226  store ra 2612
227  lhi ra 10
228  or ra 60 # get pixel x
229  store ra 2613
230  call 1994
231  load ra 2614
232  store ra 2617
233  lhi ra 10
234  or ra 163 # get pixel y
235  store ra 2613
236  call 1994
237  load ra 2614
238  store ra 2618
239  load ra 2722 # check x pos
240  subm ra 2617
241  jumpnz 248
242  load ra 2825 # check y pos
243  subm ra 2618
244  jumpnz 248
245  move ra 0xff
246  store ra 2616 # set 2616 flag
247  jump 353 # snake hit snake 2616
248  load ra 2612 # inc 2114
249  add ra 1
250  store ra 2612
251  subm ra 2721 # is 2114 > tail pointer
252  jumpnz 227
253  load ra 2722 # has snake hit ede boundary
254  sub ra 8
255  jumpc 270
256  load ra 2722
257  lhi rb 0
258  or rb 148
259  sub rb ra
260  jumpc 270
261  load ra 2825
262  sub ra 20
263  jumpc 270
264  load ra 2825
265  lhi rb 0
266  or rb 111
267  sub rb ra
268  jumpc 270
269  jump 273
270  move ra 0xff
271  store ra 2616 # set 2616 flag
272  jump 353 # snake hit edge 2616
273  lhi ra 10
274  or ra 60 # move segments to next position
275  store ra 2613
276  lhi ra 10
277  or ra 161
278  store ra 2611
279  load ra 2722 # set data
280  store ra 2614
281  call 2035
282  lhi ra 10
283  or ra 163 # set list address
284  store ra 2613
285  lhi ra 11
286  or ra 8 # set tail pointer
287  store ra 2611
288  load ra 2825 # set data
289  store ra 2614
290  call 2035
291  load ra 2722 # has snake hit food, check x pos
292  subm ra 2826
293  jumpnz 300
294  load ra 2825 # has snake hit food, check y pos
295  subm ra 2827
296  jumpnz 300
297  move ra 0xff
298  store ra 2615 # set 354 eaten flag
299  jump 333 # 195 display
300  lhi ra 10
301  or ra 60 # get tail x pos
302  store ra 2613
303  lhi ra 10
304  or ra 161
305  store ra 2611
306  call 1999
307  load ra 2614
308  store ra 2617
309  lhi ra 10
310  or ra 163 # get tail y pos
311  store ra 2613
312  lhi ra 11
313  or ra 8
314  store ra 2611
315  call 1999
316  load ra 2614
317  store ra 2618
318  call 398 # clear old tail position pixel
319  lhi ra 10
320  or ra 60
321  store ra 2613
322  lhi ra 10
323  or ra 161
324  store ra 2611
325  call 2027 # remove old x position from list
326  lhi ra 10
327  or ra 163
328  store ra 2613
329  lhi ra 11
330  or ra 8
331  store ra 2611
332  call 2027 # remove old y position from list
333  move ra 0 # 195 pixels
334  store ra 2612
335  lhi ra 10
336  or ra 60 # get pixel x
337  store ra 2613
338  call 1994
339  load ra 2614
340  store ra 2617
341  lhi ra 10
342  or ra 163 # get pixel y
343  store ra 2613
344  call 1994
345  load ra 2614
346  store ra 2618
347  call 401 # 406 snake segment
348  load ra 2612 # inc 2114
349  add ra 1
350  store ra 2612
351  subm ra 2721 # is 2114 > index
352  jumpnz 335
353  ret
354  lhi ra 0
355  or ra 148
356  store ra 2407
357  call 2079
358  load ra 2407
359  store ra 2826
360  lhi ra 0
361  or ra 112
362  store ra 2407
363  call 2079
364  load ra 2407
365  store ra 2827
366  move ra 0 # check that new position is not in snake
367  store ra 2612
368  lhi ra 10
369  or ra 60 # get pixel x
370  store ra 2613
371  call 1994
372  load ra 2614
373  store ra 2617
374  lhi ra 10
375  or ra 163 # get pixel y
376  store ra 2613
377  call 1994
378  load ra 2614
379  store ra 2618
380  load ra 2826 # check x pos
381  subm ra 2617
382  jumpnz 387
383  load ra 2827 # check y pos
384  subm ra 2618
385  jumpnz 387
386  jump 354 # 354 in snake redo
387  load ra 2612 # inc 2114
388  add ra 1
389  store ra 2612
390  subm ra 2721 # is 2114 > tail pointer
391  jumpnz 368
392  load ra 2826 # 406 354
393  store ra 2617
394  load ra 2827
395  store ra 2618
396  call 404
397  ret
398  move ra 0x00
399  store ra 2116
400  jump 406
401  move ra 0x0c
402  store ra 2116
403  jump 406
404  move ra 0x30
405  store ra 2116
406  load ra 2617
407  store ra 2117
408  load ra 2618
409  store ra 2119
410  load ra 2617
411  add ra 2
412  store ra 2118
413  load ra 2618
414  add ra 3
415  store ra 2120
416  call 1384
417  ret
418  move ra 0x00
419  store ra 2116
420  lhi ra 6
421  or ra 148
422  store ra 2121
423  lhi ra 0
424  or ra 71
425  store ra 2122
426  lhi ra 0
427  or ra 92
428  store ra 2123
429  call 1374
430  move ra 0x2a
431  store ra 2116
432  move ra 45
433  store ra 2117
434  move ra 60
435  store ra 2119
436  lhi rd 38
437  or rd 156
438  call 1291
439  move ra 0x2a
440  store ra 2116
441  move ra 53
442  store ra 2117
443  move ra 60
444  store ra 2119
445  lhi rd 38
446  or rd 132
447  call 1291
448  move ra 0x2a
449  store ra 2116
450  move ra 61
451  store ra 2117
452  move ra 60
453  store ra 2119
454  lhi rd 38
455  or rd 180
456  call 1291
457  move ra 0x2a
458  store ra 2116
459  move ra 69
460  store ra 2117
461  move ra 60
462  store ra 2119
463  lhi rd 38
464  or rd 148
465  call 1291
466  move ra 0x2a
467  store ra 2116
468  move ra 77
469  store ra 2117
470  move ra 60
471  store ra 2119
472  lhi rd 38
473  or rd 0
474  call 1291
475  move ra 0x2a
476  store ra 2116
477  move ra 87
478  store ra 2117
479  move ra 60
480  store ra 2119
481  lhi rd 38
482  or rd 188
483  call 1291
484  move ra 0x2a
485  store ra 2116
486  move ra 95
487  store ra 2117
488  move ra 60
489  store ra 2119
490  lhi rd 38
491  or rd 216
492  call 1291
493  move ra 0x2a
494  store ra 2116
495  move ra 103
496  store ra 2117
497  move ra 60
498  store ra 2119
499  lhi rd 38
500  or rd 148
501  call 1291
502  move ra 0x2a
503  store ra 2116
504  move ra 111
505  store ra 2117
506  move ra 60
507  store ra 2119
508  lhi rd 38
509  or rd 200
510  call 1291
511  ret
512  move rb ra
513  move ra 0
514  store ra 2128
515  store ra 2129
516  store ra 2130
517  sub rb 100
518  move rc rb
519  rol rc
520  and rc 1
521  jumpnz 526
522  load ra 2130
523  add ra 1
524  store ra 2130
525  jump 517
526  add rb 100
527  sub rb 10
528  move rc rb
529  rol rc
530  and rc 1
531  jumpnz 536
532  load ra 2129
533  add ra 1
534  store ra 2129
535  jump 527
536  add rb 10
537  move ra rb
538  store ra 2128
539  ret
540  load ra 2619
541  add ra 1
542  store ra 2619
543  call 512
544  load ra 2128
545  and ra 0xff
546  jumpz 566
547  sub ra 1
548  jumpz 576
549  sub ra 1
550  jumpz 586
551  sub ra 1
552  jumpz 596
553  sub ra 1
554  jumpz 606
555  sub ra 1
556  jumpz 616
557  sub ra 1
558  jumpz 626
559  sub ra 1
560  jumpz 636
561  sub ra 1
562  jumpz 646
563  sub ra 1
564  jumpz 656
565  jump 566
566  move ra 0x2a
567  store ra 2116
568  move ra 59
569  store ra 2117
570  move ra 9
571  store ra 2119
572  lhi rd 38
573  or rd 64
574  call 1291
575  jump 666
576  move ra 0x2a
577  store ra 2116
578  move ra 59
579  store ra 2117
580  move ra 9
581  store ra 2119
582  lhi rd 38
583  or rd 68
584  call 1291
585  jump 666
586  move ra 0x2a
587  store ra 2116
588  move ra 59
589  store ra 2117
590  move ra 9
591  store ra 2119
592  lhi rd 38
593  or rd 72
594  call 1291
595  jump 666
596  move ra 0x2a
597  store ra 2116
598  move ra 59
599  store ra 2117
600  move ra 9
601  store ra 2119
602  lhi rd 38
603  or rd 76
604  call 1291
605  jump 666
606  move ra 0x2a
607  store ra 2116
608  move ra 59
609  store ra 2117
610  move ra 9
611  store ra 2119
612  lhi rd 38
613  or rd 80
614  call 1291
615  jump 666
616  move ra 0x2a
617  store ra 2116
618  move ra 59
619  store ra 2117
620  move ra 9
621  store ra 2119
622  lhi rd 38
623  or rd 84
624  call 1291
625  jump 666
626  move ra 0x2a
627  store ra 2116
628  move ra 59
629  store ra 2117
630  move ra 9
631  store ra 2119
632  lhi rd 38
633  or rd 88
634  call 1291
635  jump 666
636  move ra 0x2a
637  store ra 2116
638  move ra 59
639  store ra 2117
640  move ra 9
641  store ra 2119
642  lhi rd 38
643  or rd 92
644  call 1291
645  jump 666
646  move ra 0x2a
647  store ra 2116
648  move ra 59
649  store ra 2117
650  move ra 9
651  store ra 2119
652  lhi rd 38
653  or rd 96
654  call 1291
655  jump 666
656  move ra 0x2a
657  store ra 2116
658  move ra 59
659  store ra 2117
660  move ra 9
661  store ra 2119
662  lhi rd 38
663  or rd 100
664  call 1291
665  jump 666
666  load ra 2129
667  and ra 0xff
668  jumpz 688
669  sub ra 1
670  jumpz 698
671  sub ra 1
672  jumpz 708
673  sub ra 1
674  jumpz 718
675  sub ra 1
676  jumpz 728
677  sub ra 1
678  jumpz 738
679  sub ra 1
680  jumpz 748
681  sub ra 1
682  jumpz 758
683  sub ra 1
684  jumpz 768
685  sub ra 1
686  jumpz 778
687  jump 688
688  move ra 0x2a
689  store ra 2116
690  move ra 51
691  store ra 2117
692  move ra 9
693  store ra 2119
694  lhi rd 38
695  or rd 64
696  call 1291
697  jump 788
698  move ra 0x2a
699  store ra 2116
700  move ra 51
701  store ra 2117
702  move ra 9
703  store ra 2119
704  lhi rd 38
705  or rd 68
706  call 1291
707  jump 788
708  move ra 0x2a
709  store ra 2116
710  move ra 51
711  store ra 2117
712  move ra 9
713  store ra 2119
714  lhi rd 38
715  or rd 72
716  call 1291
717  jump 788
718  move ra 0x2a
719  store ra 2116
720  move ra 51
721  store ra 2117
722  move ra 9
723  store ra 2119
724  lhi rd 38
725  or rd 76
726  call 1291
727  jump 788
728  move ra 0x2a
729  store ra 2116
730  move ra 51
731  store ra 2117
732  move ra 9
733  store ra 2119
734  lhi rd 38
735  or rd 80
736  call 1291
737  jump 788
738  move ra 0x2a
739  store ra 2116
740  move ra 51
741  store ra 2117
742  move ra 9
743  store ra 2119
744  lhi rd 38
745  or rd 84
746  call 1291
747  jump 788
748  move ra 0x2a
749  store ra 2116
750  move ra 51
751  store ra 2117
752  move ra 9
753  store ra 2119
754  lhi rd 38
755  or rd 88
756  call 1291
757  jump 788
758  move ra 0x2a
759  store ra 2116
760  move ra 51
761  store ra 2117
762  move ra 9
763  store ra 2119
764  lhi rd 38
765  or rd 92
766  call 1291
767  jump 788
768  move ra 0x2a
769  store ra 2116
770  move ra 51
771  store ra 2117
772  move ra 9
773  store ra 2119
774  lhi rd 38
775  or rd 96
776  call 1291
777  jump 788
778  move ra 0x2a
779  store ra 2116
780  move ra 51
781  store ra 2117
782  move ra 9
783  store ra 2119
784  lhi rd 38
785  or rd 100
786  call 1291
787  jump 788
788  load ra 2130
789  and ra 0xff
790  jumpz 810
791  sub ra 1
792  jumpz 820
793  sub ra 1
794  jumpz 830
795  sub ra 1
796  jumpz 840
797  sub ra 1
798  jumpz 850
799  sub ra 1
800  jumpz 860
801  sub ra 1
802  jumpz 870
803  sub ra 1
804  jumpz 880
805  sub ra 1
806  jumpz 890
807  sub ra 1
808  jumpz 900
809  jump 810
810  move ra 0x2a
811  store ra 2116
812  move ra 43
813  store ra 2117
814  move ra 9
815  store ra 2119
816  lhi rd 38
817  or rd 64
818  call 1291
819  jump 910
820  move ra 0x2a
821  store ra 2116
822  move ra 43
823  store ra 2117
824  move ra 9
825  store ra 2119
826  lhi rd 38
827  or rd 68
828  call 1291
829  jump 910
830  move ra 0x2a
831  store ra 2116
832  move ra 43
833  store ra 2117
834  move ra 9
835  store ra 2119
836  lhi rd 38
837  or rd 72
838  call 1291
839  jump 910
840  move ra 0x2a
841  store ra 2116
842  move ra 43
843  store ra 2117
844  move ra 9
845  store ra 2119
846  lhi rd 38
847  or rd 76
848  call 1291
849  jump 910
850  move ra 0x2a
851  store ra 2116
852  move ra 43
853  store ra 2117
854  move ra 9
855  store ra 2119
856  lhi rd 38
857  or rd 80
858  call 1291
859  jump 910
860  move ra 0x2a
861  store ra 2116
862  move ra 43
863  store ra 2117
864  move ra 9
865  store ra 2119
866  lhi rd 38
867  or rd 84
868  call 1291
869  jump 910
870  move ra 0x2a
871  store ra 2116
872  move ra 43
873  store ra 2117
874  move ra 9
875  store ra 2119
876  lhi rd 38
877  or rd 88
878  call 1291
879  jump 910
880  move ra 0x2a
881  store ra 2116
882  move ra 43
883  store ra 2117
884  move ra 9
885  store ra 2119
886  lhi rd 38
887  or rd 92
888  call 1291
889  jump 910
890  move ra 0x2a
891  store ra 2116
892  move ra 43
893  store ra 2117
894  move ra 9
895  store ra 2119
896  lhi rd 38
897  or rd 96
898  call 1291
899  jump 910
900  move ra 0x2a
901  store ra 2116
902  move ra 43
903  store ra 2117
904  move ra 9
905  store ra 2119
906  lhi rd 38
907  or rd 100
908  call 1291
909  jump 910
910  ret
911  move rb 0x0f
912  jump 926
913  move rb 0x1f
914  jump 926
915  move rb 0x3f
916  jump 926
917  move rb 0x7f
918  jump 926
919  move rb 0xff
920  and rb 0xff
921  jump 926
922  lhi rb 1
923  or rb 255
924  jump 926
925  move rb 0xff
926  lhi ra 15
927  or ra 255
928  sub ra 1
929  jumpnz 928
930  sub rb 1
931  jumpnz 926
932  ret
933  move rb rd
934  add rb 10
935  load ra (rb)
936  rol ra
937  rol ra
938  rol ra
939  rol ra
940  rol ra
941  rol ra
942  rol ra
943  rol ra
944  and ra 255
945  store ra 0xff5
946  store ra 2365
947  load ra (rb)
948  and ra 255
949  store ra 0xff6
950  store ra 2367
951  ret
952  move rb rd
953  add rb 14
954  load ra (rb)
955  rol ra
956  rol ra
957  rol ra
958  rol ra
959  rol ra
960  rol ra
961  rol ra
962  rol ra
963  and ra 0xff
964  store ra 2378
965  load ra (rb)
966  and ra 0xff
967  store ra 2377
968  move rb rd
969  add rb 15
970  load ra (rb)
971  rol ra
972  rol ra
973  rol ra
974  rol ra
975  rol ra
976  rol ra
977  rol ra
978  rol ra
979  and ra 0xff
980  store ra 2380
981  load ra (rb)
982  and ra 0xff
983  store ra 2379
984  ret
985  move rb rd
986  add rb 11
987  load ra (rb)
988  rol ra
989  rol ra
990  rol ra
991  rol ra
992  rol ra
993  rol ra
994  rol ra
995  rol ra
996  and ra 255
997  store ra 2363
998  and ra 0x80
999  jumpz 1003
1000  load ra 2111
1001  addm ra 2363
1002  store ra 2363
1003  load ra (rb)
1004  and ra 255
1005  store ra 2364
1006  and ra 0x80
1007  jumpz 1011
1008  load ra 2111
1009  addm ra 2364
1010  store ra 2364
1011  ret
1012  move rb rd
1013  add rb 9
1014  load ra (rb)
1015  and ra 255
1016  store ra 0xff3
1017  ret
1018  move rb rd
1019  add rb 9
1020  load ra (rb)
1021  rol ra
1022  rol ra
1023  rol ra
1024  rol ra
1025  rol ra
1026  rol ra
1027  rol ra
1028  rol ra
1029  and ra 255
1030  store ra 0xff3
1031  ret
1032  move rb rd
1033  add rb 1
1034  load rc (rd)
1035  and rc 0x7
1036  add rb rc
1037  load ra (rd)
1038  and ra 16
1039  jumpz 1046
1040  load ra (rd)
1041  and ra 0xf8
1042  add rc 1
1043  and rc 0x7
1044  add ra rc
1045  store ra (rd)
1046  load ra (rb)
1047  store ra 0xff1
1048  store ra 2362
1049  ret
1050  move rb rd
1051  add rb 1
1052  load rc (rd)
1053  and rc 0x7
1054  add rb rc
1055  load ra (rb)
1056  store ra 0xff1
1057  store ra 2362
1058  ret
1059  move rb rd
1060  add rb 12
1061  load ra (rb)
1062  and ra 0xff
1063  store ra 2369
1064  load ra (rb)
1065  rol ra
1066  rol ra
1067  rol ra
1068  rol ra
1069  rol ra
1070  rol ra
1071  rol ra
1072  rol ra
1073  and ra 0xff
1074  store ra 2371
1075  move rb rd
1076  add rb 13
1077  load ra (rb)
1078  and ra 0xff
1079  store ra 2373
1080  load ra (rb)
1081  rol ra
1082  rol ra
1083  rol ra
1084  rol ra
1085  rol ra
1086  rol ra
1087  rol ra
1088  rol ra
1089  and ra 0xff
1090  store ra 2375
1091  ret
1092  load rc (rd)
1093  and rc 32
1094  jumpz 1099
1095  move ra 0x08
1096  store ra 2358
1097  call 1018
1098  jump 1102
1099  move ra 0x06
1100  store ra 2358
1101  call 1012
1102  call 1050
1103  call 1122
1104  ret
1105  load rc (rd)
1106  and rc 32
1107  jumpz 1110
1108  call 1018
1109  jump 1111
1110  call 1012
1111  move ra 0x08
1112  store ra 2358
1113  call 1050
1114  call 1122
1115  ret
1116  move ra 0x08
1117  store ra 2358
1118  call 1018
1119  call 1050
1120  call 1122
1121  ret
1122  call 933
1123  load ra 2358
1124  store ra 0xff0
1125  lhi ra 15
1126  or ra 240
1127  load rb (ra)
1128  and rb 0x02
1129  jumpz 1125
1130  ret
1131  ret
1132  call 933
1133  call 985
1134  call 952
1135  call 1059
1136  load ra 2365
1137  addm ra 2363
1138  store ra 2365
1139  load ra 2367
1140  addm ra 2364
1141  store ra 2367
1142  load ra (rd)
1143  and ra 8
1144  jumpz 1152
1145  load rc (rd)
1146  add rc 1
1147  and rc 0x7
1148  load ra (rd)
1149  and ra 0xf8
1150  add ra rc
1151  store ra (rd)
1152  load ra 2363
1153  jumpc 1167
1154  load ra 2365
1155  addm ra 2371
1156  subm ra 2378
1157  jumpc 1179
1158  load ra 2365
1159  subm ra 2363
1160  store ra 2365
1161  load ra (rd)
1162  and ra 16
1163  jumpz 1179
1164  move ra 0xff
1165  store ra 2363
1166  jump 1179
1167  load ra 2365
1168  addm ra 2369
1169  subm ra 2377
1170  jumpc 1179
1171  load ra 2365
1172  subm ra 2363
1173  store ra 2365
1174  load ra (rd)
1175  and ra 16
1176  jumpz 1179
1177  move ra 0x01
1178  store ra 2363
1179  load ra 2364
1180  jumpc 1194
1181  load ra 2367
1182  addm ra 2375
1183  subm ra 2380
1184  jumpc 1206
1185  load ra 2367
1186  subm ra 2364
1187  store ra 2367
1188  load ra (rd)
1189  and ra 16
1190  jumpz 1206
1191  move ra 0xff
1192  store ra 2364
1193  jump 1206
1194  load ra 2367
1195  addm ra 2373
1196  subm ra 2379
1197  jumpc 1206
1198  load ra 2367
1199  subm ra 2364
1200  store ra 2367
1201  load ra (rd)
1202  and ra 16
1203  jumpz 1206
1204  move ra 0x01
1205  store ra 2364
1206  move rc rd
1207  add rc 10
1208  load ra 2367
1209  move rb ra
1210  and rb 0xff
1211  load ra 2365
1212  and ra 0xff
1213  rol ra
1214  rol ra
1215  rol ra
1216  rol ra
1217  rol ra
1218  rol ra
1219  rol ra
1220  rol ra
1221  add rb ra
1222  store rb (rc)
1223  move rc rd
1224  add rc 11
1225  load ra 2364
1226  move rb ra
1227  and rb 0xff
1228  load ra 2363
1229  and ra 0xff
1230  rol ra
1231  rol ra
1232  rol ra
1233  rol ra
1234  rol ra
1235  rol ra
1236  rol ra
1237  rol ra
1238  add rb ra
1239  store rb (rc)
1240  ret
1241  load ra (rd)
1242  and ra 32
1243  jumpz 1290
1244  load ra 2360
1245  move rd ra
1246  load ra (rd)
1247  and ra 32
1248  jumpz 1290
1249  call 933
1250  load ra 2365
1251  store ra 2366
1252  load ra 2367
1253  store ra 2368
1254  call 1059
1255  load ra 2369
1256  store ra 2370
1257  load ra 2371
1258  store ra 2372
1259  load ra 2373
1260  store ra 2374
1261  load ra 2375
1262  store ra 2376
1263  load ra 2359
1264  move rd ra
1265  call 933
1266  call 1059
1267  load ra 2365
1268  addm ra 2371
1269  subm ra 2366
1270  subm ra 2370
1271  jumpc 1290
1272  load ra 2366
1273  addm ra 2372
1274  subm ra 2365
1275  subm ra 2369
1276  jumpc 1290
1277  load ra 2367
1278  addm ra 2375
1279  subm ra 2368
1280  subm ra 2374
1281  jumpc 1290
1282  load ra 2368
1283  addm ra 2376
1284  subm ra 2367
1285  subm ra 2373
1286  jumpc 1290
1287  load ra 2361
1288  or ra 1
1289  store ra 2361
1290  ret
1291  load ra 2116
1292  store ra 0xff3 # colour
1293  load ra 2117 # x
1294  store ra 0xff5
1295  load ra 2119 # y
1296  store ra 0xff6
1297  move ra rd
1298  store ra 0xff1 # tile addr in video memory
1299  move ra 0x06
1300  store ra 0xff0 # 0
1301  lhi ra 15
1302  or ra 240
1303  load rb (ra)
1304  and rb 0x02
1305  jumpz 1301
1306  ret
1307  load ra 2116
1308  store ra 0xff3 # colour
1309  load ra 2117 # x
1310  store ra 0xff5
1311  load ra 2119 # y
1312  store ra 0xff6
1313  move ra rd
1314  store ra 0xff1 # tile addr in video memory
1315  move ra 0x08
1316  store ra 0xff0 # 0
1317  lhi ra 15
1318  or ra 240
1319  load rb (ra)
1320  and rb 0x02
1321  jumpz 1317
1322  ret
1323  lhi rd 38
1324  or rd 64 # base address
1325  load ra 2127 # get offset
1326  and ra 0x0f
1327  move rb ra
1328  sub rb 9 # <= 9
1329  jumpc 1334
1330  asl ra # x 4
1331  asl ra
1332  add rd ra # add offset to base address
1333  call 1291
1334  ret
1335  lhi rd 38
1336  or rd 0 # base address space (0x20)
1337  load ra 2126 # get ascii value
1338  and ra 0xff
1339  move rb ra
1340  sub rb 0x7f # <= 0x7f
1341  jumpc 1334
1342  asl ra # x 4
1343  asl ra
1344  add rd ra # add offset to base address
1345  call 1291
1346  ret
1347  load ra 2125 # get ascii value
1348  move rd ra
1349  call 1291
1350  ret
1351  lhi rb 15
1352  or rb 243
1353  load ra 2116
1354  store ra (rb)
1355  lhi rb 15
1356  or rb 242
1357  load ra 2121
1358  addm ra 2122
1359  store ra (rb)
1360  lhi rb 15
1361  or rb 244
1362  load ra 2121
1363  store ra (rb)
1364  lhi ra 15
1365  or ra 240
1366  move rb 0x05
1367  store rb (ra)
1368  lhi ra 15
1369  or ra 240
1370  load rb (ra)
1371  and rb 0x02
1372  jumpz 1368
1373  ret
1374  call 1351
1375  load ra 2123
1376  sub ra 1
1377  store ra 2123
1378  jumpz 1383
1379  load ra 2121
1380  add ra 80
1381  store ra 2121
1382  jump 1374
1383  ret
1384  load ra 2117
1385  store ra 0xff5
1386  load ra 2119
1387  store ra 0xff6
1388  load ra 0xff3
1389  store ra 2121
1390  load ra 2118
1391  subm ra 2117
1392  asr ra
1393  store ra 2122
1394  call 1351
1395  load ra 2120
1396  subm ra 2119
1397  jumpz 1402
1398  load ra 2119
1399  add ra 1
1400  store ra 2119
1401  jump 1384
1402  ret
1403  lhi rb 15
1404  or rb 242
1405  load ra 2121
1406  addm ra 2122
1407  store ra (rb)
1408  lhi rb 15
1409  or rb 241
1410  load ra 2121
1411  store ra (rb)
1412  lhi rb 15
1413  or rb 244
1414  load ra 2124
1415  store ra (rb)
1416  lhi ra 15
1417  or ra 240
1418  move rb 0x04
1419  store rb (ra)
1420  lhi ra 15
1421  or ra 240
1422  load rb (ra)
1423  and rb 0x02
1424  jumpz 1420
1425  ret
1426  ret
1427  call 1403
1428  load ra 2123
1429  sub ra 1
1430  store ra 2123
1431  jumpz 1439
1432  load ra 2121
1433  add ra 80
1434  store ra 2121
1435  load ra 2124
1436  add ra 80
1437  store ra 2124
1438  jump 1427
1439  ret
1440  load ra 2131 # get source address
1441  move rb ra
1442  load ra 2132 # get destination address
1443  move rc ra
1444  load ra (rb) # load char
1445  and ra 0xff
1446  jumpz 1451 # exit if 0
1447  store ra (rc) # copy
1448  add rb 1
1449  add rc 1
1450  jump 1444 # repeat
1451  ret
1452  lhi rb 8
1453  or rb 88 # get string address
1454  load ra (rb) # load char
1455  and ra 0xff
1456  jumpz 1463 # exit if 0
1457  store ra 0xfff # tx
1458  load ra 0xffe # 20 tx status wait till 1
1459  and ra 0x04
1460  jumpz 1458
1461  add rb 1 # inc address
1462  jump 1454 # repeat
1463  move ra 0
1464  store ra 2133
1465  ret
1466  load ra 2133
1467  lhi rc 8
1468  or rc 88
1469  add rc ra
1470  move ra 0
1471  store ra (rc)
1472  ret
1473  load ra 2133
1474  lhi rc 8
1475  or rc 88
1476  add rc ra
1477  add ra 2
1478  store ra 2133
1479  move ra 0x0a
1480  store ra (rc)
1481  add rc 1
1482  move ra 0x0d
1483  store ra (rc)
1484  ret
1485  load ra 2133
1486  lhi rc 8
1487  or rc 88
1488  add rc ra
1489  add ra 1
1490  store ra 2133
1491  move ra 0x20
1492  store ra (rc)
1493  ret
1494  load ra 2125
1495  asr ra
1496  asr ra
1497  asr ra
1498  asr ra
1499  asr ra
1500  asr ra
1501  asr ra
1502  asr ra
1503  store ra 2126
1504  call 1510
1505  load ra 2125
1506  and ra 0xff
1507  store ra 2126
1508  call 1510
1509  ret
1510  load ra 2133
1511  lhi rc 8
1512  or rc 88
1513  add rc ra
1514  add ra 2
1515  store ra 2133
1516  lhi rb 8
1517  or rb 218
1518  load ra 2126
1519  and ra 0xf0
1520  asr ra
1521  asr ra
1522  asr ra
1523  asr ra
1524  add rb ra
1525  load ra (rb)
1526  store ra (rc)
1527  add rc 1
1528  lhi rb 8
1529  or rb 218
1530  load ra 2126
1531  and ra 0x0f
1532  add rb ra
1533  load ra (rb)
1534  store ra (rc)
1535  ret
1536  load ra 2133
1537  lhi rc 8
1538  or rc 88
1539  add rc ra
1540  add ra 3
1541  store ra 2133
1542  load ra 2126
1543  and ra 0xff
1544  move rb 0
1545  sub ra 100 # sub 100
1546  jumpc 1549 # yes, exit
1547  add rb 1 # inc 2114
1548  jump 1545 # repeat
1549  add rb 0x30
1550  store rb (rc)
1551  add rc 1
1552  add ra 100 # undo last sub
1553  move rb 0
1554  sub ra 10 # sub 10
1555  jumpc 1558 # yes, exit
1556  add rb 1 # inc 2114
1557  jump 1554 # repeat
1558  add rb 0x30
1559  store rb (rc)
1560  add rc 1
1561  add ra 10 # undo last sub
1562  add ra 0x30
1563  store ra (rc)
1564  ret
1565  load ra 0xffe # has ascii char been rx?
1566  and ra 0x01
1567  jumpz 1565 # no check again
1568  load ra 0xfff
1569  store ra 0xfff
1570  load ra 0xffe # 20 tx status wait till 1
1571  and ra 0x04
1572  jumpz 1570
1573  jump 1565 # repeat
1574  move ra 0
1575  store ra 2133
1576  lhi ra 8
1577  or ra 248 # set string address
1578  store ra 2131
1579  lhi ra 8
1580  or ra 88 # set buffer address
1581  store ra 2132
1582  call 1440
1583  call 1452
1584  ret
1585  load ra 0xffd # set enable 2115
1586  add ra 32
1587  store ra 0xffc
1588  call 1651
1589  load ra 0xffd # clear enable 2115
1590  sub ra 32
1591  store ra 0xffc
1592  call 1651
1593  ret
1594  move ra 0x08
1595  store ra 0xffc
1596  call 1651
1597  call 1585
1598  move ra 0x00
1599  store ra 0xffc
1600  call 1651
1601  call 1585
1602  ret
1603  move ra 0x0c
1604  store ra 0xffc
1605  call 1651
1606  call 1585
1607  move ra 0x00
1608  store ra 0xffc
1609  call 1651
1610  call 1585
1611  ret
1612  move ra 0 # clear all bits
1613  store ra 0xffc
1614  load ra 2389
1615  and ra 0xff
1616  jumpnz 1620
1617  load ra 0xffd # set rs bit (text)
1618  add ra 16
1619  store ra 0xffc
1620  load ra 2388
1621  move rb ra
1622  load ra (rb) # exit if null
1623  and ra 0xff
1624  jumpz 1650
1625  and ra 0xf0 # extract high 2127
1626  ror ra
1627  ror ra
1628  ror ra
1629  ror ra
1630  addm ra 0xffd
1631  store ra 0xffc
1632  call 1651
1633  call 1585
1634  load ra (rb)
1635  and ra 0x0f # extract low 2127
1636  move rb ra
1637  load ra 0xffd
1638  and ra 0xf0
1639  add ra rb
1640  store ra 0xffc
1641  call 1651
1642  call 1585
1643  move ra 0
1644  store ra 0xffc
1645  call 1651
1646  load ra 2388
1647  add ra 1
1648  store ra 2388
1649  jump 1612
1650  ret
1651  lhi ra 2
1652  or ra 0
1653  store ra 2387
1654  load ra 2387
1655  sub ra 1
1656  store ra 2387
1657  jumpnz 1654
1658  ret
1659  move rb 0
1660  load ra 0xffb # move?
1661  and ra 0x04
1662  jumpz 1670
1663  store ra 0xffb # reset flag
1664  load ra 0xffb # left/right
1665  and ra 0x02
1666  jumpz 1669
1667  move rb 1
1668  jump 1670
1669  move rb 2
1670  move ra rb
1671  store ra 2383
1672  ret
1673  move rb 0
1674  load ra 0xffb # fire pressed
1675  and ra 0x01
1676  jumpz 1678
1677  move rb 1
1678  move ra rb
1679  store ra 2384
1680  ret
1681  move ra 0 # 0 snes 195
1682  store ra 0xff9
1683  load ra 0xff9 # wait for data
1684  and ra 0x01
1685  jumpnz 1683
1686  load ra 0xffa # read data
1687  store ra 2385
1688  ret
1689  lhi rb 255
1690  or rb 0
1691  load ra 0xff8
1692  and ra rb
1693  jumpz 1698
1694  load ra 0xff8
1695  store ra 0xff8
1696  and ra 0xff
1697  store ra 2386
1698  ret
1699  load ra 2319
1700  store ra 0xff5
1701  load ra 2320
1702  store ra 0xff6
1703  lhi rb 15
1704  or rb 243
1705  load ra 2318
1706  store ra (rb)
1707  lhi ra 15
1708  or ra 240
1709  move rb 0x07
1710  store rb (ra)
1711  lhi ra 15
1712  or ra 240
1713  load rb (ra)
1714  and rb 0x02
1715  jumpz 1711
1716  ret
1717  load ra 2326 # 20 if x1 > x0
1718  subm ra 2324
1719  move rb ra
1720  load ra 2113
1721  and ra rb
1722  jumpnz 1727
1723  load ra 2326 # x1 > x0
1724  subm ra 2324
1725  store ra 2328 # calc dx
1726  jump 1730
1727  load ra 2324 # x0 > x1
1728  subm ra 2326
1729  store ra 2328 # calc dx
1730  ret
1731  load ra 2327 # 20 if y1 > y0
1732  subm ra 2325
1733  move rb ra
1734  load ra 2113
1735  and ra rb
1736  jumpnz 1741
1737  load ra 2327 # y1 > y0
1738  subm ra 2325
1739  store ra 2329 # calc dy
1740  jump 1744
1741  load ra 2325 # y0 > y2
1742  subm ra 2327
1743  store ra 2329 # calc dy
1744  ret
1745  call 1717
1746  call 1731
1747  move ra 0
1748  store ra 2332
1749  load ra 2328 # is dy > dx
1750  subm ra 2329
1751  jumpc 1753
1752  jump 1767
1753  move ra 1
1754  store ra 2332
1755  load ra 2325 # x0,y0 = y0,x0
1756  move rb ra
1757  load ra 2324
1758  store ra 2325
1759  move ra rb
1760  store ra 2324
1761  load ra 2327 # x1,y1 = y1,x1
1762  move rb ra
1763  load ra 2326
1764  store ra 2327
1765  move ra rb
1766  store ra 2326
1767  load ra 2326 # x0 > x1?
1768  subm ra 2324
1769  jumpc 1771
1770  jump 1784
1771  move rc 0xff
1772  load ra 2324 # x0,x1 = x1,x0
1773  move rb ra
1774  load ra 2326
1775  store ra 2324
1776  move ra rb
1777  store ra 2326
1778  load ra 2325 # y0,y1 = y1,y0
1779  move rb ra
1780  load ra 2327
1781  store ra 2325
1782  move ra rb
1783  store ra 2327
1784  call 1717
1785  call 1731
1786  load ra 2328
1787  asr ra
1788  store ra 2330
1789  load ra 2325
1790  store ra 2323
1791  load ra 2324
1792  store ra 2322
1793  move ra 1
1794  store ra 2331
1795  load ra 2325
1796  subm ra 2327
1797  jumpc 1800
1798  move ra 0xff
1799  store ra 2331
1800  load ra 2321 # set 2115 colour
1801  store ra 2318
1802  load ra 2332 # is dy > dx
1803  and ra 0xff
1804  jumpnz 1806
1805  jump 1811
1806  load ra 2323 # set xy
1807  store ra 2319
1808  load ra 2322
1809  store ra 2320
1810  jump 1815
1811  load ra 2322 # set xy
1812  store ra 2319
1813  load ra 2323
1814  store ra 2320
1815  call 1699 # plot xy
1816  load ra 2330
1817  subm ra 2329
1818  store ra 2330
1819  jumpc 1821
1820  jump 1827
1821  load ra 2323
1822  addm ra 2331
1823  store ra 2323
1824  load ra 2330
1825  addm ra 2328
1826  store ra 2330
1827  load ra 2322
1828  add ra 1
1829  store ra 2322
1830  load ra 2326
1831  subm ra 2322
1832  jumpc 1834
1833  jump 1802
1834  ret
1835  load ra 2339
1836  store ra 2335
1837  move ra 0
1838  store ra 2336
1839  store ra 2340
1840  load ra 2333 # set 2115 colour
1841  store ra 2321
1842  store ra 2318
1843  load ra 2335
1844  subm ra 2336
1845  jumpc 1993
1846  load ra 2337
1847  addm ra 2335
1848  store ra 2319
1849  store ra 2341
1850  load ra 2338
1851  addm ra 2336
1852  store ra 2320
1853  store ra 2342
1854  load ra 2334
1855  jumpnz 1857
1856  call 1699 # plot((x0 + x, y0 + y)) a
1857  load ra 2337
1858  addm ra 2336
1859  store ra 2319
1860  store ra 2343
1861  load ra 2338
1862  addm ra 2335
1863  store ra 2320
1864  store ra 2344
1865  load ra 2334
1866  jumpnz 1868
1867  call 1699 # plot((x0 + y, y0 + x)) b
1868  load ra 2337
1869  subm ra 2336
1870  store ra 2319
1871  store ra 2345
1872  load ra 2338
1873  addm ra 2335
1874  store ra 2320
1875  store ra 2346
1876  load ra 2334
1877  jumpnz 1879
1878  call 1699 # plot((x0 - y, y0 + x)) c
1879  load ra 2337
1880  subm ra 2335
1881  store ra 2319
1882  store ra 2347
1883  load ra 2338
1884  addm ra 2336
1885  store ra 2320
1886  store ra 2348
1887  load ra 2334
1888  jumpnz 1890
1889  call 1699 # plot((x0 - x, y0 + y)) d
1890  load ra 2337
1891  subm ra 2335
1892  store ra 2319
1893  store ra 2349
1894  load ra 2338
1895  subm ra 2336
1896  store ra 2320
1897  store ra 2350
1898  load ra 2334
1899  jumpnz 1901
1900  call 1699 # plot((x0 - x, y0 - y)) e
1901  load ra 2337
1902  subm ra 2336
1903  store ra 2319
1904  store ra 2351
1905  load ra 2338
1906  subm ra 2335
1907  store ra 2320
1908  store ra 2352
1909  load ra 2334
1910  jumpnz 1912
1911  call 1699 # plot((x0 - y, y0 - x)) f
1912  load ra 2337
1913  addm ra 2336
1914  store ra 2319
1915  store ra 2353
1916  load ra 2338
1917  subm ra 2335
1918  store ra 2320
1919  store ra 2354
1920  load ra 2334
1921  jumpnz 1923
1922  call 1699 # plot((x0 + y, y0 - x)) g
1923  load ra 2337
1924  addm ra 2335
1925  store ra 2319
1926  store ra 2355
1927  load ra 2338
1928  subm ra 2336
1929  store ra 2320
1930  store ra 2356
1931  load ra 2334
1932  jumpnz 1935
1933  call 1699 # plot((x0 + x, y0 - y)) h
1934  jump 1971
1935  load ra 2341
1936  store ra 2324
1937  load ra 2342
1938  store ra 2325
1939  load ra 2347
1940  store ra 2326
1941  load ra 2348
1942  store ra 2327
1943  call 1745
1944  load ra 2343
1945  store ra 2324
1946  load ra 2344
1947  store ra 2325
1948  load ra 2345
1949  store ra 2326
1950  load ra 2346
1951  store ra 2327
1952  call 1745
1953  load ra 2355
1954  store ra 2324
1955  load ra 2356
1956  store ra 2325
1957  load ra 2349
1958  store ra 2326
1959  load ra 2350
1960  store ra 2327
1961  call 1745
1962  load ra 2353
1963  store ra 2324
1964  load ra 2354
1965  store ra 2325
1966  load ra 2351
1967  store ra 2326
1968  load ra 2352
1969  store ra 2327
1970  call 1745
1971  load ra 2336
1972  add ra 1
1973  store ra 2336
1974  addm ra 2336
1975  addm ra 2340
1976  add ra 1
1977  store ra 2340
1978  load ra 2340
1979  subm ra 2335
1980  add ra ra
1981  add ra 1
1982  jumpc 1843
1983  load ra 2335
1984  sub ra 1
1985  store ra 2335
1986  addm ra 2335
1987  move rb ra
1988  load ra 2340
1989  add ra 1
1990  sub ra rb
1991  store ra 2340
1992  jump 1843
1993  ret
1994  load ra 2612 # get index
1995  addm ra 2613 # add base addr
1996  load ra (ra) # read data
1997  store ra 2614 # store to buffer
1998  ret
1999  load ra 2611 # get pntr address
2000  load ra (ra) # read index
2001  sub ra 1 # dec ptr to last written addr
2002  addm ra 2613 # add base addr
2003  load ra (ra) # read data
2004  store ra 2614 # store to buffer
2005  ret
2006  load ra 2611 # get pntr address
2007  load ra (ra) # load index
2008  sub ra 0 # is list empty?
2009  jumpz 2026 # yes
2010  load ra 2613
2011  move rc ra # rc=dst
2012  add ra 1 # rb=src
2013  move rb ra
2014  load ra (rb) # load data
2015  store ra (rc) # shuffle data up
2016  add rb 1 # move pointers
2017  add rc 1
2018  load ra 2611 # reached end of list?
2019  sub ra rb # if src = tail 2616
2020  jumpnz 2014
2021  load ra 2611 # load tail ptr
2022  move rb ra # buffer address
2023  load ra (rb) # load index
2024  sub ra 1 # inc index
2025  store ra (rb) # 195 index
2026  ret
2027  load ra 2611 # get pntr address
2028  move rb ra # buffer pntr
2029  load ra (rb) # read index
2030  sub ra 0 # is index 0?
2031  jumpz 2034 # yes, exit
2032  sub ra 1 # no, decrement
2033  store ra (rb) # store index
2034  ret
2035  load ra 2611 # load index addr
2036  load ra (ra) # load index
2037  move rb ra # buffer index
2038  addm ra 2613 # generate read address
2039  move rc ra # rc=dst
2040  sub rb 0 # is list empty?
2041  jumpz 2055 # yes
2042  lhi ra 0
2043  or ra 100 # max index
2044  sub ra rb
2045  jumpz 2062 # 0=full 0!=space
2046  move rb rc
2047  sub rb 1 # rb=src
2048  load ra (rb) # load data
2049  store ra (rc) # shuffle data down
2050  sub rb 1 # move pointers
2051  sub rc 1
2052  load ra 2613 # read head of list addr
2053  sub ra rc # if dst = head 2616
2054  jumpnz 2048
2055  load ra 2614 # write data to head of list
2056  store ra (rc)
2057  load ra 2611 # load tail pointer
2058  move rb ra # buffer address
2059  load ra (rb) # load index
2060  add ra 1 # inc index
2061  store ra (rb) # 195 index
2062  ret
2063  load ra 2611 # load index addr
2064  load ra (ra) # load index
2065  addm ra 2613 # generate read address
2066  move rc ra # rc=dst
2067  lhi ra 0
2068  or ra 100 # max index
2069  sub ra rb
2070  jumpz 2078 # 0=full 0!=space
2071  load ra 2614
2072  store ra (rc)
2073  load ra 2611 # inc tail pointer
2074  move rb ra
2075  load ra (ra) #
2076  add ra 1 #
2077  store ra (rb)
2078  ret
2079  load ra 2407 # base number (8bit)
2080  move rc ra #
2081  lhi ra 9
2082  or ra 105 # get base address
2083  addm ra 2408 # add index
2084  load rb (ra) # read 2079 number (0-255)
2085  mul rc rb # mul
2086  asr rc # /256
2087  asr rc
2088  asr rc
2089  asr rc
2090  asr rc
2091  asr rc
2092  asr rc
2093  asr rc
2094  load ra 2112
2095  and ra rc
2096  store ra 2407
2097  sub ra 20
2098  jumpc 2100
2099  jump 2102
2100  move ra 24
2101  store ra 2407
2102  lhi rb 0
2103  or rb 200 # is index > list size
2104  load ra 2408
2105  sub rb ra
2106  jumpnz 2108
2107  move ra 0xff # set index -1
2108  add ra 1 # inc index
2109  store ra 2408
2110  ret
2111  .data 0xff00
2112  .data 0xfffc
2113  .data 0x8000
2114  .data 0
2115  .data 0
2116  .data 0
2117  .data 0
2118  .data 0
2119  .data 0
2120  .data 0
2121  .data 0
2122  .data 0
2123  .data 0
2124  .data 0
2125  .data 0
2126  .data 0
2127  .data 0
2128  .data 0
2129  .data 0
2130  .data 0
2131  .data 0x00
2132  .data 0x00
2133  .data 0x00
2134  .data 0x00
2135  .data 0x00
2136  .data 0
2137  .data 0
2138  .data 0
2139  .data 0
2140  .data 0
2141  .data 0
2142  .data 0
2143  .data 0
2144  .data 0
2145  .data 0
2146  .data 0
2147  .data 0
2148  .data 0
2149  .data 0
2150  .data 0
2151  .data 0
2152  .data 0
2153  .data 0
2154  .data 0
2155  .data 0
2156  .data 0
2157  .data 0
2158  .data 0
2159  .data 0
2160  .data 0
2161  .data 0
2162  .data 0
2163  .data 0
2164  .data 0
2165  .data 0
2166  .data 0
2167  .data 0
2168  .data 0
2169  .data 0
2170  .data 0
2171  .data 0
2172  .data 0
2173  .data 0
2174  .data 0
2175  .data 0
2176  .data 0
2177  .data 0
2178  .data 0
2179  .data 0
2180  .data 0
2181  .data 0
2182  .data 0
2183  .data 0
2184  .data 0
2185  .data 0
2186  .data 0
2187  .data 0
2188  .data 0
2189  .data 0
2190  .data 0
2191  .data 0
2192  .data 0
2193  .data 0
2194  .data 0
2195  .data 0
2196  .data 0
2197  .data 0
2198  .data 0
2199  .data 0
2200  .data 0
2201  .data 0
2202  .data 0
2203  .data 0
2204  .data 0
2205  .data 0
2206  .data 0
2207  .data 0
2208  .data 0
2209  .data 0
2210  .data 0
2211  .data 0
2212  .data 0
2213  .data 0
2214  .data 0
2215  .data 0
2216  .data 0
2217  .data 0
2218  .data 0
2219  .data 0
2220  .data 0
2221  .data 0
2222  .data 0
2223  .data 0
2224  .data 0
2225  .data 0
2226  .data 0
2227  .data 0
2228  .data 0
2229  .data 0
2230  .data 0
2231  .data 0
2232  .data 0
2233  .data 0
2234  .data 0
2235  .data 0
2236  .data 0
2237  .data 0
2238  .data 0
2239  .data 0
2240  .data 0
2241  .data 0
2242  .data 0
2243  .data 0
2244  .data 0
2245  .data 0
2246  .data 0
2247  .data 0
2248  .data 0
2249  .data 0
2250  .data 0
2251  .data 0
2252  .data 0
2253  .data 0
2254  .data 0
2255  .data 0
2256  .data 0
2257  .data 0
2258  .data 0
2259  .data 0
2260  .data 0
2261  .data 0
2262  .data 0
2263  .data 0
2264  .data 0
2265  .data 0
2266  .data 0x30 # '0'
2267  .data 0x31 # '1'
2268  .data 0x32 # '2'
2269  .data 0x33 # '3'
2270  .data 0x34 # '4'
2271  .data 0x35 # '5'
2272  .data 0x36 # '6'
2273  .data 0x37 # '7'
2274  .data 0x38 # '8'
2275  .data 0x39 # '9'
2276  .data 0x41 # 'a'
2277  .data 0x42 # 'b'
2278  .data 0x43 # 'c'
2279  .data 0x44 # 'd'
2280  .data 0x45 # 'e'
2281  .data 0x46 # 'f'
2282  .data 0x48 # h
2283  .data 0x65 # e
2284  .data 0x6c # l
2285  .data 0x6c # l
2286  .data 0x6f # 0
2287  .data 0x20 # space
2288  .data 0x57 # w
2289  .data 0x6f # o
2290  .data 0x72 # r
2291  .data 0x6c # l
2292  .data 0x64 # d
2293  .data 0x0a # lf
2294  .data 0x0d # cr
2295  .data 0x00 # null
2296  .data 0x0a # lf
2297  .data 0x0d # cr
2298  .data 0x53 # s
2299  .data 0x69 # i
2300  .data 0x6d # m
2301  .data 0x70 # p
2302  .data 0x6c # l
2303  .data 0x65 # e
2304  .data 0x43 # c
2305  .data 0x50 # p
2306  .data 0x55 # u
2307  .data 0x20 # space
2308  .data 0x44 # d
2309  .data 0x65 # e
2310  .data 0x62 # b
2311  .data 0x75 # u
2312  .data 0x67 # g
2313  .data 0x0a # lf
2314  .data 0x0d # cr
2315  .data 0x00 # null
2316  .data 0
2317  .data 0
2318  .data 0
2319  .data 0
2320  .data 0
2321  .data 0
2322  .data 0
2323  .data 0
2324  .data 0
2325  .data 0
2326  .data 0
2327  .data 0
2328  .data 0
2329  .data 0
2330  .data 0
2331  .data 0
2332  .data 0
2333  .data 0
2334  .data 0
2335  .data 0
2336  .data 0
2337  .data 0
2338  .data 0
2339  .data 0
2340  .data 0
2341  .data 0
2342  .data 0
2343  .data 0
2344  .data 0
2345  .data 0
2346  .data 0
2347  .data 0
2348  .data 0
2349  .data 0
2350  .data 0
2351  .data 0
2352  .data 0
2353  .data 0
2354  .data 0
2355  .data 0
2356  .data 0
2357  .data 0
2358  .data 0
2359  .data 0
2360  .data 0
2361  .data 0
2362  .data 0
2363  .data 0
2364  .data 0
2365  .data 0
2366  .data 0
2367  .data 0
2368  .data 0
2369  .data 0
2370  .data 0
2371  .data 0
2372  .data 0
2373  .data 0
2374  .data 0
2375  .data 0
2376  .data 0
2377  .data 0
2378  .data 0
2379  .data 0
2380  .data 0
2381  .data 0
2382  .data 0
2383  .data 0
2384  .data 0
2385  .data 0
2386  .data 0
2387  .data 0
2388  .data 0
2389  .data 0
2390  .data 0x33
2391  .data 0x32
2392  .data 0x06
2393  .data 0x0c
2394  .data 0x28
2395  .data 0x01
2396  .data 0x80
2397  .data 0
2398  .data 0x4f # o
2399  .data 0x58 # x
2400  .data 0x4f # o
2401  .data 0x20 # space
2402  .data 0x44 # d
2403  .data 0x65 # e
2404  .data 0x6d # m
2405  .data 0x6f # o
2406  .data 0
2407  .data 0
2408  .data 0
2409  .data 50
2410  .data 100
2411  .data 224
2412  .data 221
2413  .data 40
2414  .data 210
2415  .data 31
2416  .data 218
2417  .data 181
2418  .data 129
2419  .data 92
2420  .data 228
2421  .data 248
2422  .data 76
2423  .data 69
2424  .data 127
2425  .data 195
2426  .data 199
2427  .data 219
2428  .data 149
2429  .data 196
2430  .data 143
2431  .data 243
2432  .data 205
2433  .data 105
2434  .data 115
2435  .data 142
2436  .data 192
2437  .data 168
2438  .data 256
2439  .data 28
2440  .data 145
2441  .data 52
2442  .data 27
2443  .data 50
2444  .data 155
2445  .data 190
2446  .data 44
2447  .data 124
2448  .data 118
2449  .data 251
2450  .data 128
2451  .data 107
2452  .data 232
2453  .data 247
2454  .data 60
2455  .data 102
2456  .data 35
2457  .data 200
2458  .data 39
2459  .data 252
2460  .data 73
2461  .data 123
2462  .data 198
2463  .data 99
2464  .data 165
2465  .data 254
2466  .data 49
2467  .data 185
2468  .data 12
2469  .data 203
2470  .data 121
2471  .data 67
2472  .data 184
2473  .data 136
2474  .data 30
2475  .data 179
2476  .data 100
2477  .data 183
2478  .data 197
2479  .data 255
2480  .data 48
2481  .data 146
2482  .data 86
2483  .data 139
2484  .data 208
2485  .data 175
2486  .data 186
2487  .data 89
2488  .data 101
2489  .data 158
2490  .data 32
2491  .data 156
2492  .data 242
2493  .data 238
2494  .data 207
2495  .data 225
2496  .data 11
2497  .data 62
2498  .data 154
2499  .data 37
2500  .data 163
2501  .data 138
2502  .data 103
2503  .data 135
2504  .data 94
2505  .data 239
2506  .data 15
2507  .data 125
2508  .data 236
2509  .data 217
2510  .data 66
2511  .data 216
2512  .data 202
2513  .data 24
2514  .data 106
2515  .data 180
2516  .data 172
2517  .data 77
2518  .data 18
2519  .data 14
2520  .data 63
2521  .data 53
2522  .data 78
2523  .data 176
2524  .data 204
2525  .data 26
2526  .data 234
2527  .data 84
2528  .data 25
2529  .data 34
2530  .data 85
2531  .data 171
2532  .data 194
2533  .data 170
2534  .data 206
2535  .data 116
2536  .data 64
2537  .data 148
2538  .data 222
2539  .data 249
2540  .data 241
2541  .data 51
2542  .data 229
2543  .data 144
2544  .data 152
2545  .data 45
2546  .data 161
2547  .data 240
2548  .data 157
2549  .data 20
2550  .data 117
2551  .data 80
2552  .data 23
2553  .data 90
2554  .data 131
2555  .data 134
2556  .data 108
2557  .data 191
2558  .data 160
2559  .data 46
2560  .data 93
2561  .data 36
2562  .data 91
2563  .data 233
2564  .data 147
2565  .data 230
2566  .data 58
2567  .data 231
2568  .data 56
2569  .data 110
2570  .data 42
2571  .data 57
2572  .data 226
2573  .data 174
2574  .data 167
2575  .data 72
2576  .data 83
2577  .data 214
2578  .data 119
2579  .data 223
2580  .data 55
2581  .data 98
2582  .data 97
2583  .data 133
2584  .data 173
2585  .data 79
2586  .data 59
2587  .data 95
2588  .data 178
2589  .data 193
2590  .data 82
2591  .data 126
2592  .data 215
2593  .data 75
2594  .data 21
2595  .data 151
2596  .data 253
2597  .data 112
2598  .data 13
2599  .data 114
2600  .data 54
2601  .data 209
2602  .data 245
2603  .data 81
2604  .data 104
2605  .data 38
2606  .data 227
2607  .data 68
2608  .data 19
2609  .data 132
2610  .data 0
2611  .data 0
2612  .data 0
2613  .data 0
2614  .data 0
2615  .data 0
2616  .data 0
2617  .data 0
2618  .data 0
2619  .data 0
2620  .data 0
2621  .data 0
2622  .data 0
2623  .data 0
2624  .data 0
2625  .data 0
2626  .data 0
2627  .data 0
2628  .data 0
2629  .data 0
2630  .data 0
2631  .data 0
2632  .data 0
2633  .data 0
2634  .data 0
2635  .data 0
2636  .data 0
2637  .data 0
2638  .data 0
2639  .data 0
2640  .data 0
2641  .data 0
2642  .data 0
2643  .data 0
2644  .data 0
2645  .data 0
2646  .data 0
2647  .data 0
2648  .data 0
2649  .data 0
2650  .data 0
2651  .data 0
2652  .data 0
2653  .data 0
2654  .data 0
2655  .data 0
2656  .data 0
2657  .data 0
2658  .data 0
2659  .data 0
2660  .data 0
2661  .data 0
2662  .data 0
2663  .data 0
2664  .data 0
2665  .data 0
2666  .data 0
2667  .data 0
2668  .data 0
2669  .data 0
2670  .data 0
2671  .data 0
2672  .data 0
2673  .data 0
2674  .data 0
2675  .data 0
2676  .data 0
2677  .data 0
2678  .data 0
2679  .data 0
2680  .data 0
2681  .data 0
2682  .data 0
2683  .data 0
2684  .data 0
2685  .data 0
2686  .data 0
2687  .data 0
2688  .data 0
2689  .data 0
2690  .data 0
2691  .data 0
2692  .data 0
2693  .data 0
2694  .data 0
2695  .data 0
2696  .data 0
2697  .data 0
2698  .data 0
2699  .data 0
2700  .data 0
2701  .data 0
2702  .data 0
2703  .data 0
2704  .data 0
2705  .data 0
2706  .data 0
2707  .data 0
2708  .data 0
2709  .data 0
2710  .data 0
2711  .data 0
2712  .data 0
2713  .data 0
2714  .data 0
2715  .data 0
2716  .data 0
2717  .data 0
2718  .data 0
2719  .data 0
2720  .data 0
2721  .data 0
2722  .data 0
2723  .data 0
2724  .data 0
2725  .data 0
2726  .data 0
2727  .data 0
2728  .data 0
2729  .data 0
2730  .data 0
2731  .data 0
2732  .data 0
2733  .data 0
2734  .data 0
2735  .data 0
2736  .data 0
2737  .data 0
2738  .data 0
2739  .data 0
2740  .data 0
2741  .data 0
2742  .data 0
2743  .data 0
2744  .data 0
2745  .data 0
2746  .data 0
2747  .data 0
2748  .data 0
2749  .data 0
2750  .data 0
2751  .data 0
2752  .data 0
2753  .data 0
2754  .data 0
2755  .data 0
2756  .data 0
2757  .data 0
2758  .data 0
2759  .data 0
2760  .data 0
2761  .data 0
2762  .data 0
2763  .data 0
2764  .data 0
2765  .data 0
2766  .data 0
2767  .data 0
2768  .data 0
2769  .data 0
2770  .data 0
2771  .data 0
2772  .data 0
2773  .data 0
2774  .data 0
2775  .data 0
2776  .data 0
2777  .data 0
2778  .data 0
2779  .data 0
2780  .data 0
2781  .data 0
2782  .data 0
2783  .data 0
2784  .data 0
2785  .data 0
2786  .data 0
2787  .data 0
2788  .data 0
2789  .data 0
2790  .data 0
2791  .data 0
2792  .data 0
2793  .data 0
2794  .data 0
2795  .data 0
2796  .data 0
2797  .data 0
2798  .data 0
2799  .data 0
2800  .data 0
2801  .data 0
2802  .data 0
2803  .data 0
2804  .data 0
2805  .data 0
2806  .data 0
2807  .data 0
2808  .data 0
2809  .data 0
2810  .data 0
2811  .data 0
2812  .data 0
2813  .data 0
2814  .data 0
2815  .data 0
2816  .data 0
2817  .data 0
2818  .data 0
2819  .data 0
2820  .data 0
2821  .data 0
2822  .data 0
2823  .data 0
2824  .data 0
2825  .data 0
2826  .data 0
2827  .data 0
