# =============================================================================================================
# *
# * Copyright (c) Mike
# *
# * File Name: simpleCPUv1d.m4
# *
# * Version: V1.0
# *
# * Release Date:
# *
# * Author(s): M.Freeman
# *
# * Description: macros
# *
# * Conditions of Use: THIS CODE IS COPYRIGHT AND IS SUPPLIED "AS IS" WITHOUT WARRANTY OF ANY KIND, INCLUDING,
# *                    BUT NOT LIMITED TO, ANY IMPLIED WARRANTY OF MERCHANTABILITY AND FITNESS FOR A
# *                    PARTICULAR PURPOSE.
# *
# * Notes:
# *
# =============================================================================================================

#    unary + and -
#    ** or ^ (exponentiation)
#    * / % (modulus)
#    + -
#    == != < <= > >=
#    !(not)
#    & or && (logical AND)
#    | or || (logical OR)

# ------
# - IO -
# ------
# memory map
# ----------
#                          3210
# CE0 : UART   : 1111 1111 111X
# CE1 : GPIO A : 1111 1111 110X
# CE2 : ROTARY : 1111 1111 101X
# CE3 : SNES   : 1111 1111 100X
# CE4 : PS2    : 1111 1111 011X
# CE5 : NU     : 1111 1111 010X
# CE6 : VGA    : 1111 1111 0XXX

define(CPU_CTL_REG, `0xFF0')
define(CPU_STATUS_REG, `0xFF0')
define(CPU_SRC_ADDR_REG, `0xFF1')
define(CPU_DATA_IN_REG, `0xFF2')
define(CPU_DATA_OUT_REG, `0xFF2')
define(CPU_COLOUR_REG, `0xFF3')
define(TILE_POS, `0xFF3')
define(CPU_DST_ADDR_REG, `0xFF4')
define(TILE_X_REG, `0xFF5')
define(TILE_Y_REG, `0xFF6')
define(PIXEL_X_REG, `0xFF5')
define(PIXEL_Y_REG, `0xFF6')

define(PS2_DATA, `0xFF8')
define(PS2_RESET, `0xFF8')

define(SNES_START, `0xFF9')
define(SNES_STATUS, `0xFF9')
define(SNES_DATA, `0xFFA')

define(ROTARY_DATA, `0xFFB')
define(ROTARY_RESET, `0xFFB')

define(GPIO_A_IN, `0xFFC')
define(GPIO_A_OUT,`0xFFC')
define(GPIO_A_DATA, `0xFFD')

define(UART_STATUS, `0xFFE')
define(UART_RX, `0xFFF')
define(UART_TX, `0xFFF')

# --------
# - GAME -
# --------

define(SNAKE_TOP_LIMIT, `20')
define(SNAKE_BOTTOM_LIMIT, `111')
define(SNAKE_LEFT_LIMIT, `8')
define(SNAKE_RIGHT_LIMIT, `148')
define(SNAKE_LENGTH, `6')

define(PONG_X_MIN, `14')
define(PONG_X_MAX, `142')
define(PONG_Y_MIN, `21')
define(PONG_Y_MAX, `105')

define(WINDOW_WIDTH_DIV_2, `80')
define(WINDOW_HEIGHT_DIV_2, `60')

# ------------
# - CONSTANT -
# ------------

define(WIDTH, `148')
define(HEIGHT, `112')

define(MAX_RND_LIST_SIZE, `200')
define(MAX_LIST_SIZE, `100')

define(VDC_MAX_X_LIMIT, `160')
define(VDC_MID_X_LIMIT, `80')

define(VDC_MAX_Y_LIMIT, `120')
define(VDC_MID_Y_LIMIT, `60')

define(VDC_MIN_X_LIMIT, `8')
define(VDC_MIN_Y_LIMIT, `16')

define(VDC_MIN_ADDRESS, `0')
define(VDC_MAX_ADDRESS, `eval( ((160 * 120)/2) - 1) ')

define(DOWN, `1')
define(UP, `255')
define(LEFT, `255')
define(RIGHT, `1')
define(STOP, `0')

define(TITLE_X_POSITION, `45')
define(LIVES_X_POSITION, `83')

define(SCORE_X_POSITION, `3')
define(SCORE_Y_POSITION, `9')

define( INTRO_LINE1_X_POSITION,`10')
define( INTRO_LINE1_Y_POSITION,`40')

define( INTRO_LINE2_X_POSITION,`50')
define( INTRO_LINE2_Y_POSITION,`55')

define( INTRO_LINE3_X_POSITION,`40')
define( INTRO_LINE3_Y_POSITION,`70')

define( GAME_OVER_X_POSITION,`45')
define( GAME_OVER_Y_POSITION,`60')

define( ANY, `0xFF' )
define( ALL, `0xFF' )

define( UPPER_BYTE_MASK, `0xFF00' )
define( LOWER_BYTE_MASK, `0x00FF' )

# -----------
# - COLOURS -
# -----------

#             XXRR GGBB 
# RED         0011 0000 = 0x30
# GREEN       0000 1100 = 0x0C
# BLUE        0000 0011 = 0x03
# YELLOW      0011 1100 = 0x3C
# ORANGE      0011 0100 = 0x34
# TURQUOISE   0000 1111 = 0x0F
# LIGHT_BLUE  0000 0111 = 0x07
# LIGHT_GREEN 0000 1101 = 0x0D
# LIME_GREEN  0001 1100 = 0x1C
# PURPLE      0001 0011 = 0x13
# PINK        0011 0011 = 0x33
# SCARLET     0011 0001 = 0x31
# GREY        0001 0101 = 0x15
# LIGHT_GREY  0010 1010 = 0x2A
# BLACK       0000 0000 = 0x00
# WHITE       0011 1111 = 0xFF

define(RED, `0x30')
define(GREEN, `0x0C')
define(BLUE, `0x03')
define(YELLOW, `0x3C')
define(ORANGE, `0x34')
define(TURQUOISE, `0x0F')
define(LIGHT_BLUE, `0x07')
define(LIGHT_GREEN, `0x0D')
define(LIME_GREEN, `0x1C')
define(PURPLE, `0x13')
define(PINK, `0x33')
define(SCARLET, `0x31')
define(GREY, `0x15')
define(LIGHT_GREY, `0x2A')
define(BLACK, `0x00')
define(WHITE, `0xFF')

# -----------------------
# - VIDEO COMMAND CODES -
# -----------------------

define(WRITE_PIXEL, `0x00')
define(WRITE_PIXEL_AUTO_INC, `0x01')
define(READ_PIXEL, `0x02')
define(READ_PIXEL_AUTO_INC, `0x03')
define(BLOCK_COPY, `0x04')
define(BLOCK_WRITE, `0x05')
define(DRAW_TILE, `0x06')
define(DRAW_PIXEL, `0x07')
define(OVERLAY_TILE, `0x08')

define(VID_MASK, `0x04')
define(CTL_MASK, `0x02')
define(RAM_MASK, `0x01')

# ----------------------------
# - CHARACTER TILE ADDRESSES -
# ----------------------------

define(sp_CHAR, `eval( 9728 + 0*4 )')
define(!_CHAR, `eval( 9600 + 1*4 )')
define(doubleQuote_CHAR, `eval( 9728 + 2*4 )')
define(hash_CHAR, `eval( 9728 + 3*4 )')
define($_CHAR, `eval( 9728 + 4*4 )')
define(%_CHAR, `eval( 9728 + 5*4 )')
define(&_CHAR, `eval( 9728 + 6*4 )')
define('_CHAR, `eval( 9728 + 7*4 )')
define((_CHAR, `eval( 9728 + 8*4 )')      
define()_CHAR, `eval( 9728 + 9*4 )')
define(asterix_CHAR, `eval( 9728 + 10*4 )')
define(+_CHAR, `eval( 9728 + 11*4 )')
define(comma_CHAR, `eval( 9728 + 12*4 )')
define(-_CHAR, `eval( 9728 + 13*4 )')
define(._CHAR, `eval( 9728 + 14*4 )')
define(/_CHAR, `eval( 9728 + 15*4 )')

define(zero_CHAR, `eval( 9728 + 16*4 )')
define(one_CHAR, `eval( 9728 + 17*4 )')
define(two_CHAR, `eval( 9728 + 18*4 )')
define(three_CHAR, `eval( 9728 + 19*4 )')
define(four_CHAR, `eval( 9728 + 20*4 )')
define(five_CHAR, `eval( 9728 + 21*4 )')
define(six_CHAR, `eval( 9728 + 22*4 )')
define(seven_CHAR, `eval( 9728 + 23*4 )')
define(eight_CHAR, `eval( 9728 + 24*4 )')
define(nine_CHAR, `eval( 9728 + 25*4 )')

define(colon_CHAR, `eval( 9728 + 26*4 )')
define(;_CHAR, `eval( 9728 + 27*4 )')
define(<_CHAR, `eval( 9728 + 28*4 )')
define(=_CHAR, `eval( 9728 + 29*4 )')  
define(>_CHAR, `eval( 9728 + 30*4 )')
define(?_CHAR, `eval( 9728 + 31*4 )')
define(@_CHAR, `eval( 9728 + 32*4 )')
define(A_CHAR, `eval( 9728 + 33*4 )')
define(B_CHAR, `eval( 9728 + 34*4 )')
define(C_CHAR, `eval( 9728 + 35*4 )')
define(D_CHAR, `eval( 9728 + 36*4 )')
define(E_CHAR, `eval( 9728 + 37*4 )')
define(F_CHAR, `eval( 9728 + 38*4 )')
define(G_CHAR, `eval( 9728 + 39*4 )')
define(H_CHAR, `eval( 9728 + 40*4 )')
define(I_CHAR, `eval( 9728 + 41*4 )')
define(J_CHAR, `eval( 9728 + 42*4 )')
define(K_CHAR, `eval( 9728 + 43*4 )')
define(L_CHAR, `eval( 9728 + 44*4 )')
define(M_CHAR, `eval( 9728 + 45*4 )')
define(N_CHAR, `eval( 9728 + 46*4 )')
define(O_CHAR, `eval( 9728 + 47*4 )')
define(P_CHAR, `eval( 9728 + 48*4 )')
define(Q_CHAR, `eval( 9728 + 49*4 )')
define(R_CHAR, `eval( 9728 + 50*4 )')
define(S_CHAR, `eval( 9728 + 51*4 )')
define(T_CHAR, `eval( 9728 + 52*4 )')
define(U_CHAR, `eval( 9728 + 53*4 )')
define(V_CHAR, `eval( 9728 + 54*4 )')
define(W_CHAR, `eval( 9728 + 55*4 )')
define(X_CHAR, `eval( 9728 + 56*4 )')
define(Y_CHAR, `eval( 9728 + 57*4 )')
define(Z_CHAR, `eval( 9728 + 58*4 )')
define([_CHAR, `eval( 9728 + 59*4 )')
define(\_CHAR, `eval( 9728 + 60*4 )')
define(]_CHAR, `eval( 9728 + 61*4 )')
define(^_CHAR, `eval( 9728 + 62*4 )')
define(underscore_CHAR, `eval( 9728 + 63*4 )')
define(quote_CHAR, `eval( 9728 + 64*4 )')
define(a_CHAR, `eval( 9728 + 65*4 )')
define(b_CHAR, `eval( 9728 + 66*4 )')
define(c_CHAR, `eval( 9728 + 67*4 )')
define(d_CHAR, `eval( 9728 + 68*4 )')
define(e_CHAR, `eval( 9728 + 69*4 )')
define(f_CHAR, `eval( 9728 + 70*4 )')
define(g_CHAR, `eval( 9728 + 71*4 )')
define(h_CHAR, `eval( 9728 + 72*4 )')
define(i_CHAR, `eval( 9728 + 73*4 )')
define(j_CHAR, `eval( 9728 + 74*4 )')
define(k_CHAR, `eval( 9728 + 75*4 )')
define(l_CHAR, `eval( 9728 + 76*4 )')
define(m_CHAR, `eval( 9728 + 77*4 )')
define(n_CHAR, `eval( 9728 + 78*4 )')
define(o_CHAR, `eval( 9728 + 79*4 )')
define(p_CHAR, `eval( 9728 + 80*4 )')
define(q_CHAR, `eval( 9728 + 81*4 )')
define(r_CHAR, `eval( 9728 + 82*4 )')
define(s_CHAR, `eval( 9728 + 83*4 )')
define(t_CHAR, `eval( 9728 + 84*4 )')
define(u_CHAR, `eval( 9728 + 85*4 )')
define(v_CHAR, `eval( 9728 + 86*4 )')
define(w_CHAR, `eval( 9728 + 87*4 )')
define(x_CHAR, `eval( 9728 + 88*4 )')
define(y_CHAR, `eval( 9728 + 89*4 )')
define(z_CHAR, `eval( 9728 + 90*4 )')
define({_CHAR, `eval( 9728 + 91*4 )')
define(|_CHAR, `eval( 9728 + 92*4 )')
define(}_CHAR, `eval( 9728 + 93*4 )')
define(~_CHAR, `eval( 9728 + 94*4 )')

# -----------------------------------------
# - SPACE INVADERS GRAPHIC TILE ADDRESSES -
# -----------------------------------------

define(SPRITE_BLANK, `101108' )

define(SPRITE_ALIEN_1_DOWN, `10112' )
define(SPRITE_ALIEN_1_UP, `10116' )

define(SPRITE_ALIEN_2_DOWN, `10120' )
define(SPRITE_ALIEN_2_UP, `10124' )

define(SPRITE_ALIEN_3_DOWN, `10128' )
define(SPRITE_ALIEN_3_UP, `10132' )

define(SPRITE_ALIEN_4_DOWN, `10136' )
define(SPRITE_ALIEN_4_UP, `10140' )

define(SPRITE_ALIEN_5_DOWN, `10144' )
define(SPRITE_ALIEN_5_UP, `10148' )

define(SPRITE_ALIEN_6_DOWN, `10152' )
define(SPRITE_ALIEN_6_UP, `10156' )

define(SPRITE_PLAYER, `10160' )

define(SPRITE_BASE_RIGHT, `10164' )
define(SPRITE_BASE_LEFT, `10168' )

define(SPRITE_MISSILE_1, `10172' )
define(SPRITE_MISSILE_2, `10176' )

define(SPRITE_EXPLOSION_1, `10180' )
define(SPRITE_EXPLOSION_2, `10184' )
define(SPRITE_EXPLOSION_3, `10188' )
define(SPRITE_EXPLOSION_4, `10192' )
define(SPRITE_EXPLOSION_5, `10196' )

define(SPRITE_BLANK, `10196' )

# ------------------------------
# - OXO GRAPHIC TILE ADDRESSES -
# ------------------------------

define(SQUARE_1_CHAR, `10200' )
define(SQUARE_2_CHAR, `10204' )
define(SQUARE_3_CHAR, `10208' )
define(SQUARE_4_CHAR, `10212' )

define(CORNER_1_CHAR, `10216' )
define(CORNER_2_CHAR, `10220' )
define(CORNER_3_CHAR, `10224' )
define(CORNER_4_CHAR, `10228' )

define(LINE_1_CHAR, `10232' )
define(LINE_2_CHAR, `10236' )
define(LINE_3_CHAR, `10240' )
define(LINE_4_CHAR, `10244' )

define(CORNER_5_CHAR, `10248' )
define(CORNER_6_CHAR, `10252' )
define(CORNER_7_CHAR, `10256' )
define(CORNER_8_CHAR, `10260' )

define(CORNER_9_CHAR, `10304' )
define(CORNER_10_CHAR, `10308' )
define(CORNER_11_CHAR, `10312' )
define(CORNER_12_CHAR, `10316' )

define(LINE_5_CHAR, `10292' )
define(LINE_6_CHAR, `10296' )

# -----------
# - GENERAL -
# -----------

define( goto, `jump $1')

define( run, `call $1')

# VAR VAL
# 1   2

define( set, `move RA $2
	store RA $1')

define( clr, `move RA 0
	store RA $1')

# VAR, MASK, ADDR
# 1    2     3

define( ifSet, `load RA $1
	and RA $2
	jumpnz $3')

define( ifClr, `load RA $1
	and RA $2
	jumpz $3')

# VAR, VAL, ADDR
# 1    2    3

define( ifEqual, `load RA $1
	sub RA $2
	jumpz $3')

# VAR ADDR=0 ADDR=1 ADDR=2 ...
# 1   2      3      4  

define( ifTest, `
ifelse(eval($#==2), 1, `
    load RA $1
	and RA 0xFF
	jumpz $2',)

ifelse(eval($#==3), 1, `
    load RA $1
	and RA 0xFF
	jumpz $2
    sub RA 1
	jumpz $3',)

ifelse(eval($#==4), 1, `
    load RA $1
	and RA 0xFF
	jumpz $2
    sub RA 1
	jumpz $3
    sub RA 1
	jumpz $4',)

ifelse(eval($#==5), 1, `
    load RA $1
	and RA 0xFF
	jumpz $2
    sub RA 1
	jumpz $3
    sub RA 1
	jumpz $4
    sub RA 1
	jumpz $5',)

ifelse(eval($#==6), 1, `
    load RA $1
	and RA 0xFF
	jumpz $2
    sub RA 1
	jumpz $3
    sub RA 1
	jumpz $4
    sub RA 1
	jumpz $5
    sub RA 1
	jumpz $6',)

ifelse(eval($#==7), 1, `
    load RA $1
	and RA 0xFF
	jumpz $2
    sub RA 1
	jumpz $3
    sub RA 1
	jumpz $4
    sub RA 1
	jumpz $5
    sub RA 1
	jumpz $6
    sub RA 1
	jumpz $7',)

ifelse(eval($#==8), 1, `
    load RA $1
	and RA 0xFF
	jumpz $2
    sub RA 1
	jumpz $3
    sub RA 1
	jumpz $4
    sub RA 1
	jumpz $5
    sub RA 1
	jumpz $6
    sub RA 1
	jumpz $7
    sub RA 1
	jumpz $8',)

ifelse(eval($#==9), 1, `
    load RA $1
	and RA 0xFF
	jumpz $2
    sub RA 1
	jumpz $3
    sub RA 1
	jumpz $4
    sub RA 1
	jumpz $5
    sub RA 1
	jumpz $6
    sub RA 1
	jumpz $7
    sub RA 1
	jumpz $8
    sub RA 1
	jumpz $9',)
')


# ------------------
# - GENERATE LABEL -
# ------------------

ifdef(`M4_FILE_NUM',,`define(`M4_FILE_NUM', 0)')
define(`_uniq_ix', 0)

define(`uniqlabel', `define(`_uniq_ix', incr(_uniq_ix))dnl
&&$1f`'eval(M4_FILE_NUM)_`'eval(_uniq_ix, 10, 4)')

# -----------
# - CONVERT -
# -----------

# covert_XY_2_address : convert X Y position values into pixel address
# --------------------------------------------------------------------

define( covert_XY_2_address, `lhi $1 eval( (($2+$3*160) & 0xFF00) >> 8)
    or $1 eval( ($2+$3*160) & 0xFF )')

# ---------------------------
# - RESERVE SPACE IN MEMORY -
# ---------------------------

define( space, `
    .data 0'dnl 
`ifelse( eval( $1 - 1 ), 0, `
    .data 0', `space( eval( $1 - 1 ) )')' )


define( reserve_sprite memory, `
    .data 0'dnl 
`ifelse( eval(( $1 * 6 ) - 1 ), 0, `
    .data 0', `space( eval(( $1 * 6 ) - 1 ) )')' )

# -------------
# - BIT ARRAY -
# -------------

# VARIABLES
# ---------
# RA - bit_array_data
# RB - bit_array_offset
# RC - bit_array_address


# create bit array( SIZE )
#                   1 

define( create_bit_array, `space(eval( $1 / 16 ))' )

# get_array_bit( BASE, INDEX )
#                1     2
# get_array_bit( BASE )
#                1    

define( get_array_bit, `
ifelse(eval($#==1), 1, `
    movea( RC, $1 )
    load RA bit_array_index
    move RB RA
    rotate_left( RA, 12 )
    add RC RA
    load RA (RC)
    and RB 0x0F
    call get_array_bit_subroutine',)

ifelse(eval($#==2), 1, `
    movea( RC, $1 )
    moved( RD, $2 )
    move RB RD
    rotate_left( RD, 12 )
    add RC RD
    load RA (RC)
    and RB 0x0F
    call get_array_bit_subroutine',)
')

# set_array_bit( BASE, INDEX )
#                1     2
# set_array_bit( BASE )
#                1 

define( set_array_bit, `
ifelse(eval($#==1), 1, `
    movea( RC, $1 )
    load RA bit_array_index
    move RB RA
    rotate_left( RA, 12 )
    add RC RA
    load RA (RC)
    and RB 0x0F
    call set_array_bit_subroutine',)

ifelse(eval($#==2), 1, `
    movea( RC, $1 )
    moved( RD, $2 )
    move RB RD
    rotate_left( RD, 12 )
    add RC RD
    load RA (RC)
    and RB 0x0F
    call set_array_bit_subroutine',)
')

# clr_array_bit( BASE, INDEX )
#                1     2
# clr_array_bit( BASE )
#                1 

define( clr_array_bit, `
ifelse(eval($#==1), 1, `
    movea( RC, $1 )
    load RA bit_array_index
    move RB RA
    rotate_left( RA, 12 )
    add RC RA
    load RA (RC)
    and RB 0x0F
    call clr_array_bit_subroutine',)

ifelse(eval($#==2), 1, `
    movea( RC, $1 )
    moved( RD, $2 )
    move RB RD
    rotate_left( RD, 12 )
    add RC RD
    load RA (RC)
    and RB 0x0F
    call clr_array_bit_subroutine',)
')

# -----------------------
# - MOVE ADDRESS / DATA -
# -----------------------

# movea : move 12bit address into register
# ---------------------------------------- 

define( movea, ``lhi $1 eval( ( $2 & 0xF00 ) >> 8)'
    `or $1 eval( $2 & 0xFF )'')

# moved : move 16 bit data into register
# --------------------------------------

define( moved, `lhi $1 eval( ($2 & 0xFF00) >> 8)
    or $1 eval( $2 & 0xFF )')

# --------------------
# - ROTATE AND SHIFT -
# --------------------

define( rotate_left, `
ifelse(eval($2==15), 1, `
  ror $1',)

ifelse(eval($2==14), 1, `
  ror $1
  ror $1',)

ifelse(eval($2==13), 1, `
  ror $1
  ror $1
  ror $1',)

ifelse(eval($2==12), 1, `
  ror $1
  ror $1
  ror $1
  ror $1',)

ifelse(eval($2==11), 1, `
  ror $1
  ror $1
  ror $1
  ror $1
  ror $1',)

ifelse(eval($2==10), 1, `
  ror $1
  ror $1
  ror $1
  ror $1
  ror $1
  ror $1',)

ifelse(eval($2==9), 1, `
  ror $1
  ror $1
  ror $1
  ror $1
  ror $1
  ror $1
  ror $1',)

ifelse(eval($2==8), 1, `
  rol $1
  rol $1
  rol $1
  rol $1
  rol $1
  rol $1
  rol $1
  rol $1',)

ifelse(eval($2==7), 1, `
  rol $1
  rol $1
  rol $1
  rol $1
  rol $1
  rol $1
  rol $1',)

ifelse(eval($2==6), 1, `
  rol $1
  rol $1
  rol $1
  rol $1
  rol $1
  rol $1',)

ifelse(eval($2==5), 1, `
  rol $1
  rol $1
  rol $1
  rol $1
  rol $1',)

ifelse(eval($2==4), 1, `
  rol $1
  rol $1
  rol $1
  rol $1',)
ifelse(eval($2==3), 1, `
  rol $1
  rol $1
  rol $1',)
ifelse(eval($2==2), 1, `
  rol $1
  rol $1',)
ifelse(eval($2==1), 1, `
  rol $1',)
')

# ---------
# - SLEEP -
# ---------

define( sleep, `moved( ra, $1 )
    store ra delay_high_count
    move RB RA
    call delay_start
')

# -----------
# - SPRITES -
# -----------

define(SPRITE_VISIBLE_MASK, `32' )
define(SPRITE_AUTO_MASK, `16' )
define(SPRITE_INC_MASK, `8' )
define(SPRITE_TILE_NUMBER_MASK, `0x7' )
define(SPRITE_BG_COLOUR_MASK, `255' )
define(SPRITE_FG_COLOUR_MASK, `255' )
define(SPRITE_XPOS_MASK, `255' )
define(SPRITE_YPOS_MASK, `255' )
define(SPRITE_XOFFSET_MASK, `255' )
define(SPRITE_YOFFSET_MASK, `255' )
define(SPRITE_HB_XMAX_MASK, `255' )
define(SPRITE_HB_XMIN_MASK, `255' )
define(SPRITE_HB_YMAX_MASK, `255' )
define(SPRITE_HB_YMIN_MASK, `255' )
define(SPRITE_LT_XMAX_MASK, `255' )
define(SPRITE_LT_XMIN_MASK, `255' )
define(SPRITE_LT_YMAX_MASK, `255' )
define(SPRITE_LT_YMIN_MASK, `255' )

define(SPRITE_FLAG_MASK, `0xF8' )
define(SPRITE_TURN_OFF_MASK, `0xDF' )
define(SPRITE_TURN_ON_MASK, `32' )

define(SPRITE_BG_COLOUR_SHIFT, `8' )
define(SPRITE_FG_COLOUR_SHIFT, `0' )
define(SPRITE_XPOS_SHIFT, `8' )
define(SPRITE_YPOS_SHIFT, `0' )
define(SPRITE_XOFFSET_SHIFT, `8' )
define(SPRITE_YOFFSET_SHIFT, `0' )
define(SPRITE_HB_XMAX_SHIFT, `8' )
define(SPRITE_HB_XMIN_COLOUR_SHIFT, `0' )
define(SPRITE_HB_YMAX_SHIFT, `8' )
define(SPRITE_HB_YMIN_COLOUR_SHIFT, `0' )
define(SPRITE_XHITBOX_MAX_SHIFT, `8' )
define(SPRITE_YHITBOX_MAX_SHIFT, `8' )
define(SPRITE_XLIMIT_MAX_SHIFT, `8' )
define(SPRITE_YLIMIT_MAX_SHIFT, `8' )

define(SPRITE_XYPOS_OFFSET, `10' )
define(SPRITE_XYOFFSET_OFFSET, `11' )
define(SPRITE_FB_COLOUR_OFFSET, `9' )
define(SPRITE_TILE_OFFSET, `1' )
define(SPRITE_XHITBOX_OFFSET, `12' )
define(SPRITE_YHITBOX_OFFSET, `13' )
define(SPRITE_XLIMIT_OFFSET, `14' )
define(SPRITE_YLIMIT_OFFSET, `15' )

define(SPRITE_ROW_INCREMENT, `8' )
define(SPRITE_TILE_INCREMENT, `4' )

define(SPRITE_DATA_SIZE, `16' )
define(SPRITE_WIDTH, `8' )
define(SPRITE_HEIGHT, `8' )

define(SPRITE_MAX_NUMBER, `6' )

define(SPRITE_SET_VISIBLE, `1' )
define(SPRITE_CLR_VISIBLE, `0' )
define(SPRITE_SET_AUTO, `1' )
define(SPRITE_CLR_AUTO, `0' )
define(SPRITE_SET_TILE_INC, `1' )
define(SPRITE_CLR_TILE_INC, `0' )
define(SPRITE_TILE_NUMBER_0, `0' )
define(SPRITE_TILE_NUMBER_1, `1' )
define(SPRITE_TILE_NUMBER_2, `2' )
define(SPRITE_TILE_NUMBER_3, `3' )
define(SPRITE_TILE_NUMBER_4, `4' )
define(SPRITE_TILE_NUMBER_5, `5' )
define(SPRITE_TILE_NUMBER_6, `6' )
define(SPRITE_TILE_NUMBER_7, `7' )


# VISIBLE, AUTO TILE_INC TILE_NUMBER, TILE_ADDR_0, TILE_ADDR_1, TILE_ADDR_2, TILE_ADDR_3, TILE_ADDR_4, TILE_ADDR_5, TILE_ADDR_6, TILE_ADDR_7, 
# 1        2    3        4            5            6            7            8            9            10           11           12       

# BG COLOUR, FG COLOUR, XPOS, YPOS, XOFFSET, YOFFSET, HB_XMAX, HB_XMIN, HB_YMAX, HB_YMIN, LT_XMAX, LT_XMIN, LT_YMAX, LT_YMIN,
# 13         14         15    16    17       18       19       20       21       22       23       24       25       26

# 0  : VISIBLE : AUTO : TILE_INC : TILE_NUMBER (1:1:1:3)
# 1  : TILE_ADDRESS_0
# 2  : TILE_ADDRESS_1
# 3  : TILE_ADDRESS_2
# 4  : TILE_ADDRESS_3
# 5  : TILE_ADDRESS_4
# 6  : TILE_ADDRESS_5
# 7  : TILE_ADDRESS_6
# 8  : TILE_ADDRESS_7
# 9  : BG COLOUR : FG COLOUR		(8:8)
# 10 : XPOS : YPOS					(8:8)
# 11 : XOFFSET : YOFFSET			(8:8)
# 12 : HB_XMAX : HB_XMIN			(8:8)
# 13 : HB_YMAX : HB_YMIN		    (8:8)
# 14 : LT_XMAX : LT_XMIN			(8:8)
# 15 : LT_YMAX : LT_YMIN		    (8:8)


# VISIBLE     : 0 = not visible, 1 = visible
# AUTO        : 0 = direction fixed, 1 = direction reversed (limit collisions)
# TILE_INC    : 0 = fixed tile number, 1 = incrementing tile number
# TILE_NUMBER : select tile address 0 - 7, 3bit value
# BG_COLOUR   : background colour, 8bit value
# FG_COLOUR   : foreground colour, 8bit value
# XPOS        : x position of top left pixel of tile, 8bit value
# YPOS        : y position of top left pixel of tile, 8bit value
# XOFFSET     : x offset, 8bit value
# YOFFSET     : y offset, 8bit value
# HB_XMAX     : hit box, x max, 8bit value
# HB_XMIN     : hit box, x min, 8bit value
# HB_YMAX     : hit box, y max, 8bit value
# HB_YMIN     : hit box, y min, 8bit value
# LT_XMAX     : limit, x max, 8bit value
# LT_XMIN     : limit, x min, 8bit value
# LT_YMAX     : limit, y max, 8bit value
# LT_YMIN     : limit, y min, 8bit value


#    32 16 8  421
# XX V: A: I: TTT

#
# initialize_sprite (initialise memory before run time)
# ------------------------------------------------

define( initialize_sprite, `.data eval( (( $1 & 1 ) << 5 ) | (( $2 & 1 ) << 4 ) | (( $3 & 1 ) << 3 ) | ( $4 & 7 ) )
    .data $5
    .data $6
    .data $7
    .data $8
    .data $9
    .data $10
    .data $11
    .data $12
    .data eval( ( $13 << 8 ) | $14 )
    .data eval( ( $15 << 8 ) | $16 )
    .data eval( ( $17 << 8 ) | $18 )
    .data eval( ( $19 << 8 ) | $20 )
    .data eval( ( $21 << 8 ) | $22 )
    .data eval( ( $23 << 8 ) | $24 )
    .data eval( ( $25 << 8 ) | $26 )')
    
#
# draw_sprite
# -----------
#
# SPRITE_BASE_ADDR, SPRITE_NUMBER
# 1                 2                                                     

define( draw_sprite, ``lhi RD eval( ((( $2 * 16 ) + $1 ) & 0xFF00 ) >> 8 )'
    `or RD eval( (( $2 * 16 ) + $1 ) & 0xFF )'

    move RA RD
    store RA sprite_pntr
    call draw_sprite_subroutine')

#
# overlay_sprite
# --------------
#
# SPRITE_BASE_ADDR, SPRITE_NUMBER
# 1                 2     

define( overlay_sprite, ``lhi RD eval( ((( $2 * 16 ) + $1 ) & 0xFF00 ) >> 8 )'
    `or RD eval( (( $2 * 16 ) + $1 ) & 0xFF )'

    move RA RD
    store RA sprite_pntr
    call overlay_sprite_subroutine')

# move_sprite
# -----------
#
# SPRITE_BASE_ADDR, SPRITE_NUMBER
# 1                 2     

define( move_sprite, ``lhi RD eval( ((( $2 * 16 ) + $1 ) & 0xFF00 ) >> 8 )'
    `or RD eval( (( $2 * 16 ) + $1 ) & 0xFF )'

    move RA RD
    store RA sprite_pntr
    
    call move_sprite_subroutine')

#
# clear_sprite
# -----------
#
# SPRITE_BASE_ADDR, SPRITE_NUMBER
# 1                 2     

define( clear_sprite, ``lhi RD eval( ((( $2 * 16 ) + $1 ) & 0xFF00 ) >> 8 )'
    `or RD eval( (( $2 * 16 ) + $1 ) & 0xFF )'

    move RA RD
    store RA sprite_pntr   
    call clear_sprite_subroutine')

#
# collision_sprite
# ----------------
#
# SPRITE_BASE_ADDR_A, SPRITE_NUMBER_A, SPRITE_BASE_ADDR_B, SPRITE_NUMBER_B 
# 1                   2                3                   4                     
#

define( collision_sprite, ``lhi RD eval( ((( $2 * 16 ) + $1 ) & 0xFF00 ) >> 8 )'
    `or RD eval( (( $2 * 16 ) + $1 ) & 0xFF )'

    move RA RD
    store RA sprite_pntr_1

	`lhi RD eval( ((( $4 * 16 ) + $3 ) & 0xFF00 ) >> 8 )'
    `or RD eval( (( $4 * 16 ) + $3 ) & 0xFF )'

    move RA RD
    store RA sprite_pntr

    call collision_sprite_subroutine')

#
# turn_off_sprite
# ---------------
#
# SPRITE_BASE_ADDR, SPRITE_NUMBER
# 1                 2                           
#
#    32 16 8  421
# XX V: A: I: TTT
#

define( turn_off_sprite, ``lhi RD eval( ((( $2 * 16 ) + $1 ) & 0xFF00 ) >> 8 )'
    `or RD eval( (( $2 * 16 ) + $1 ) & 0xFF )'

    load RA (RD)
	and RA 0xDF
	store RA (RD)')

#
# turn_on_sprite
# ---------------
#
# SPRITE_BASE_ADDR, SPRITE_NUMBER
# 1                 2                            
#
#    32 16 8  421
# XX V: A: I: TTT
#

define( turn_on_sprite, ``lhi RD eval( ((( $2 * 16 ) + $1 ) & 0xFF00 ) >> 8 )'
    `or RD eval( (( $2 * 16 ) + $1 ) & 0xFF )'

    load RA (RD)
	or RA 32
	store RA (RD)')

#
# set_sprite
# ----------
#
# SPRITE_BASE_ADDR, SPRITE_NUMBER, VAL
# 1                 2              3              
#
#    32 16 8  421
# XX V: A: I: TTT
#

define( set_sprite, ``lhi RD eval( ((( $2 * 16 ) + $1 ) & 0xFF00 ) >> 8 )'
    `or RD eval( (( $2 * 16 ) + $1 ) & 0xFF )'

    load RA (RD)
	and RA 0xF8
	move RB $3
	and RB 0x7
	add RA RB	
	store RA (RD)')

#
# increment_sprite
# ---------------
#
# SPRITE_BASE_ADDR, SPRITE_NUMBER
# 1                 2                            
#
#    32 16 8  421
# XX V: A: I: TTT
#

define( increment_sprite, ``lhi RD eval( ((( $2 * 16 ) + $1 ) & 0xFF00 ) >> 8 )'
    `or RD eval( (( $2 * 16 ) + $1 ) & 0xFF )'

    load RA (RD)
	and RA 0xF8
	move RB RA

    load RA (RD)
	and RA 0x7
	add RA 1
	and RA 0x7
	add RA RB	
	store RA (RD)')

#
# get_sprite
# ----------
#
# SPRITE_BASE_ADDR, SPRITE_NUMBER
# 1                 2   

define( get_sprite, ``lhi RD eval( ((( $2 * 16 ) + $1 ) & 0xFF00 ) >> 8 )'
    `or RD eval( (( $2 * 16 ) + $1 ) & 0xFF )'

    load RA (RD)
    store RA sprite
    move RA RD
    store RA sprite_pntr')

#
# position_sprite
# ---------------
#
# SPRITE_BASE_ADDR, SPRITE_NUMBER, XVAL, YVAL
# 1                 2              3     4              
#
#    32 16 8  421
# XX V: A: I: TTT
#

define( position_sprite, ``lhi RD eval( ((( $2 * 16 ) + $1 ) & 0xFF00 ) >> 8 )'
    `or RD eval( (( $2 * 16 ) + $1 ) & 0xFF )'

    add RD 10

    lhi RB $3
    
    move RC $4
    and RC 0xFF
    add RB RC    	
	store RB (RD)')

#
# offset_sprite
# -------------
#
# SPRITE_BASE_ADDR, SPRITE_NUMBER, XVAL, YVAL
# 1                 2              3     4              
#
#    32 16 8  421
# XX V: A: I: TTT
#

define( offset_sprite, ``lhi RD eval( ((( $2 * 16 ) + $1 ) & 0xFF00 ) >> 8 )'
    `or RD eval( (( $2 * 16 ) + $1 ) & 0xFF )'

    add RD 11

    lhi RB $3
    
    move RC $4
    and RC 0xFF
    add RB RC    	
	store RB (RD)')

#
# delete_sprite (at run time)
# ---------------------------

# SPRITE_BASE_ADDR, SPRITE_NUMBER
# 1                 2                

define( delete_sprite, ``lhi RD eval( ((( $2 * 16 ) + $1 ) & 0xFF00 ) >> 8 )'
    `or RD eval( (( $2 * 16 ) + $1 ) & 0xFF )'

    move RC 0
    store RC (RD)  # 0
    add RD 1
    store RC (RD)  # 1
    add RD 1
    store RC (RD)  # 2
    add RD 1
    store RC (RD)  # 3
    add RD 1
    store RC (RD)  # 4
    add RD 1
    store RC (RD)  # 5
    add RD 1
    store RC (RD)  # 6
    add RD 1
    store RC (RD)  # 7 
    add RD 1
    store RC (RD)  # 8
    add RD 1
    store RC (RD)  # 9
    add RD 1
    store RC (RD)  # 10
    add RD 1
    store RC (RD)  # 11
    add RD 1
    store RC (RD)  # 12
    add RD 1
    store RC (RD)  # 13
    add RD 1
    store RC (RD)  # 14
    add RD 1
    store RC (RD)  # 15')

#
# clear_screen
# ------------

define( clear_screen, `move RA $1
    store RA block_colour
    move RA 0
    store RA block_start_address
    moved( RA, VDC_MAX_ADDRESS )
    store RA block_width
    call block_write_subroutine')
 
#
# draw_box
# --------

# COLOUR, X_TOP, Y_TOP, X_BOTTOM, Y_BOTTOM
# 1       2      3      4         5

#             1   2  3  4  5 
#  draw_box( RED, 6, 2, 7, 3 )

define( draw_box, `move RA $1 
    store RA block_colour

    moved( RA, eval( ( ( $3 * 160 ) + $2 ) >> 1 ) )
    store RA block_start_address    

    moved( RA, eval( ( $4 - $2 ) >> 1 ) )
    store RA block_width

    moved( RA, eval( ( $5 - $3 ) + 1 ) )
    store RA block_depth

    call draw_box_subroutine')

#
# copy_box
# --------

# SRC_X_TOP, SRC_Y_TOP, SRC_X_BOTTOM, SRC_Y_BOTTOM, DEST_X_TOP, DEST_Y_TOP
# 1          2          3             4             5           6

define( copy_box, `moved( RA, eval( ( ( $2 * 160 ) + $1 ) >> 1 ) )
    store RA block_start_address    

    moved( RA, eval( ( $3 - $1 ) >> 1 ) )
    store RA block_width

    moved( RA, eval( ( $4 - $2 ) + 1 ) )
    store RA block_depth

    moved( RA, eval( ( ( $6 * 160 ) + $5 ) >> 1 ) )
    store RA block_destination_address  

    call copy_box_subroutine')

#
# draw_tile
# ---------

# COLOUR, X, Y, CHAR
# 1       2  3  4    

define( draw_tile, `move RA $1
	store RA block_colour
	move RA $2
	store RA block_x_top_pos
	move RA $3
	store RA block_y_top_pos

	`lhi RD eval( ( $4 & 0xFF00 ) >> 8 )'
    `or RD eval( $4 & 0xFF )'

    call draw_tile_subroutine')

#
# overlay_tile
# ------------

# COLOUR, X, Y, CHAR
# 1       2  3  4    

define( overlay_tile, `move RA $1
	store RA block_colour
	move RA $2
	store RA block_x_top_pos
	move RA $3
	store RA block_y_top_pos

	`lhi RD eval( ( $4 & 0xFF00 ) >> 8 )'
    `or RD eval( $4 & 0xFF )'

    call overlay_tile_subroutine')




